@@ -139,11 +139,11 @@ public final class HTMLManagerServlet extends ManagerServlet {
                         e.toString());
             }
         } else if (command.equals("/sslConnectorCiphers")) {
-            sslConnectorCiphers(request, response, smClient);
+            sslConnectorCiphers(request, response);
         } else if (command.equals("/sslConnectorCerts")) {
-            sslConnectorCerts(request, response, smClient);
+            sslConnectorCerts(request, response);
         } else if (command.equals("/sslConnectorTrustedCerts")) {
-            sslConnectorTrustedCerts(request, response, smClient);
+            sslConnectorTrustedCerts(request, response);
         } else if (command.equals("/upload") || command.equals("/deploy") ||
                 command.equals("/reload") || command.equals("/undeploy") ||
                 command.equals("/expire") || command.equals("/start") ||
@@ -185,14 +185,19 @@ public final class HTMLManagerServlet extends ManagerServlet {
         if (path != null) {
             cn = new ContextName(path, request.getParameter("version"));
         }
+
         String deployPath = request.getParameter("deployPath");
-        ContextName deployCn = null;
-        if (deployPath != null) {
-            deployCn = new ContextName(deployPath,
-                    request.getParameter("deployVersion"));
-        }
-        String deployConfig = request.getParameter("deployConfig");
         String deployWar = request.getParameter("deployWar");
+        String deployConfig = request.getParameter("deployConfig");
+        ContextName deployCn = null;
+        if (deployPath != null && deployPath.length() > 0) {
+            deployCn = new ContextName(deployPath, request.getParameter("deployVersion"));
+        } else if (deployConfig != null && deployConfig.length() > 0) {
+            deployCn = ContextName.extractFromPath(deployConfig);
+        } else if (deployWar != null && deployWar.length() > 0) {
+            deployCn = ContextName.extractFromPath(deployWar);
+        }
+
         String tlsHostName = request.getParameter("tlsHostName");
 
         // Prepare our output writer to generate the response message
@@ -368,11 +373,11 @@ public final class HTMLManagerServlet extends ManagerServlet {
         args[0] = smClient.getString("htmlManagerServlet.manager");
         args[1] = response.encodeURL(request.getContextPath() + "/html/list");
         args[2] = smClient.getString("htmlManagerServlet.list");
-        args[3] = // External link
+        args[3] = response.encodeURL
             (request.getContextPath() + "/" +
              smClient.getString("htmlManagerServlet.helpHtmlManagerFile"));
         args[4] = smClient.getString("htmlManagerServlet.helpHtmlManager");
-        args[5] = // External link
+        args[5] = response.encodeURL
             (request.getContextPath() + "/" +
              smClient.getString("htmlManagerServlet.helpManagerFile"));
         args[6] = smClient.getString("htmlManagerServlet.helpManager");
@@ -450,11 +455,9 @@ public final class HTMLManagerServlet extends ManagerServlet {
                 }
 
                 args = new Object[7];
-                args[0] = // External link
-                        "<a href=\""
-                        + URLEncoder.DEFAULT.encode(contextPath + "/", StandardCharsets.UTF_8)
-                        + "\" " + Constants.REL_EXTERNAL + ">"
-                        + Escape.htmlElementContent(displayPath) + "</a>";
+                args[0] = "<a href=\"" +
+                        URLEncoder.DEFAULT.encode(contextPath + "/", StandardCharsets.UTF_8) +
+                        "\">" + Escape.htmlElementContent(displayPath) + "</a>";
                 if ("".equals(ctxt.getWebappVersion())) {
                     args[1] = noVersion;
                 } else {
@@ -529,14 +532,15 @@ public final class HTMLManagerServlet extends ManagerServlet {
         }
 
         // Deploy Section
-        args = new Object[7];
+        args = new Object[8];
         args[0] = smClient.getString("htmlManagerServlet.deployTitle");
         args[1] = smClient.getString("htmlManagerServlet.deployServer");
         args[2] = response.encodeURL(request.getContextPath() + "/html/deploy");
         args[3] = smClient.getString("htmlManagerServlet.deployPath");
-        args[4] = smClient.getString("htmlManagerServlet.deployConfig");
-        args[5] = smClient.getString("htmlManagerServlet.deployWar");
-        args[6] = smClient.getString("htmlManagerServlet.deployButton");
+        args[4] = smClient.getString("htmlManagerServlet.deployVersion");
+        args[5] = smClient.getString("htmlManagerServlet.deployConfig");
+        args[6] = smClient.getString("htmlManagerServlet.deployWar");
+        args[7] = smClient.getString("htmlManagerServlet.deployButton");
         writer.print(MessageFormat.format(DEPLOY_SECTION, args));
 
         args = new Object[4];
@@ -760,24 +764,24 @@ public final class HTMLManagerServlet extends ManagerServlet {
 
 
     protected void sslConnectorCiphers(HttpServletRequest request,
-            HttpServletResponse response, StringManager smClient) throws ServletException, IOException {
-        request.setAttribute("cipherList", getConnectorCiphers(smClient));
+            HttpServletResponse response) throws ServletException, IOException {
+        request.setAttribute("cipherList", getConnectorCiphers());
         getServletContext().getRequestDispatcher(
                 connectorCiphersJspPath).forward(request, response);
     }
 
 
     protected void sslConnectorCerts(HttpServletRequest request,
-            HttpServletResponse response, StringManager smClient) throws ServletException, IOException {
-        request.setAttribute("certList", getConnectorCerts(smClient));
+            HttpServletResponse response) throws ServletException, IOException {
+        request.setAttribute("certList", getConnectorCerts());
         getServletContext().getRequestDispatcher(
                 connectorCertsJspPath).forward(request, response);
     }
 
 
     protected void sslConnectorTrustedCerts(HttpServletRequest request,
-            HttpServletResponse response, StringManager smClient) throws ServletException, IOException {
-        request.setAttribute("trustedCertList", getConnectorTrustedCerts(smClient));
+            HttpServletResponse response) throws ServletException, IOException {
+        request.setAttribute("trustedCertList", getConnectorTrustedCerts());
         getServletContext().getRequestDispatcher(
                 connectorTrustedCertsJspPath).forward(request, response);
     }
@@ -788,7 +792,7 @@ public final class HTMLManagerServlet extends ManagerServlet {
      */
     @Override
     public String getServletInfo() {
-        return "HTMLManagerServlet, Copyright (c) 1999-2019, The Apache Software Foundation";
+        return "HTMLManagerServlet, Copyright (c) 1999-2018, The Apache Software Foundation";
     }
 
     /**
@@ -1140,7 +1144,6 @@ public final class HTMLManagerServlet extends ManagerServlet {
                 }
             };
         }
-        //TODO: complete this to TTL, etc.
         return comparator;
     }
 
@@ -1287,7 +1290,7 @@ public final class HTMLManagerServlet extends ManagerServlet {
         "  <small>{4}</small>\n" +
         " </td>\n" +
         " <td class=\"row-left\">\n" +
-        "  <input type=\"text\" name=\"deployConfig\" size=\"20\">\n" +
+        "  <input type=\"text\" name=\"deployVersion\" size=\"20\">\n" +
         " </td>\n" +
         "</tr>\n" +
         "<tr>\n" +
@@ -1295,6 +1298,14 @@ public final class HTMLManagerServlet extends ManagerServlet {
         "  <small>{5}</small>\n" +
         " </td>\n" +
         " <td class=\"row-left\">\n" +
+        "  <input type=\"text\" name=\"deployConfig\" size=\"20\">\n" +
+        " </td>\n" +
+        "</tr>\n" +
+        "<tr>\n" +
+        " <td class=\"row-right\">\n" +
+        "  <small>{6}</small>\n" +
+        " </td>\n" +
+        " <td class=\"row-left\">\n" +
         "  <input type=\"text\" name=\"deployWar\" size=\"40\">\n" +
         " </td>\n" +
         "</tr>\n" +
@@ -1303,7 +1314,7 @@ public final class HTMLManagerServlet extends ManagerServlet {
         "  &nbsp;\n" +
         " </td>\n" +
         " <td class=\"row-left\">\n" +
-        "  <input type=\"submit\" value=\"{6}\">\n" +
+        "  <input type=\"submit\" value=\"{7}\">\n" +
         " </td>\n" +
         "</tr>\n" +
         "</table>\n" +
