@@ -66,7 +66,6 @@ import org.apache.coyote.ProtocolHandler;
 import org.apache.coyote.http11.AbstractHttp11Protocol;
 import org.apache.tomcat.util.Diagnostics;
 import org.apache.tomcat.util.ExceptionUtils;
-import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.modeler.Registry;
 import org.apache.tomcat.util.net.SSLContext;
 import org.apache.tomcat.util.net.SSLHostConfig;
@@ -312,14 +311,20 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
         String command = request.getPathInfo();
         if (command == null)
             command = request.getServletPath();
-        String config = request.getParameter("config");
+
         String path = request.getParameter("path");
+        String war = request.getParameter("war");
+        String config = request.getParameter("config");
         ContextName cn = null;
         if (path != null) {
             cn = new ContextName(path, request.getParameter("version"));
+        } else if (config != null) {
+            cn = ContextName.extractFromPath(config);
+        } else if (war != null) {
+            cn = ContextName.extractFromPath(war);
         }
+
         String type = request.getParameter("type");
-        String war = request.getParameter("war");
         String tag = request.getParameter("tag");
         boolean update = false;
         if ((request.getParameter("update") != null)
@@ -621,7 +626,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
 
     protected void sslConnectorCiphers(PrintWriter writer, StringManager smClient) {
         writer.println(smClient.getString("managerServlet.sslConnectorCiphers"));
-        Map<String,List<String>> connectorCiphers = getConnectorCiphers(smClient);
+        Map<String,List<String>> connectorCiphers = getConnectorCiphers();
         for (Map.Entry<String,List<String>> entry : connectorCiphers.entrySet()) {
             writer.println(entry.getKey());
             for (String cipher : entry.getValue()) {
@@ -634,7 +639,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
 
     private void sslConnectorCerts(PrintWriter writer, StringManager smClient) {
         writer.println(smClient.getString("managerServlet.sslConnectorCerts"));
-        Map<String,List<String>> connectorCerts = getConnectorCerts(smClient);
+        Map<String,List<String>> connectorCerts = getConnectorCerts();
         for (Map.Entry<String,List<String>> entry : connectorCerts.entrySet()) {
             writer.println(entry.getKey());
             for (String cert : entry.getValue()) {
@@ -646,7 +651,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
 
     private void sslConnectorTrustedCerts(PrintWriter writer, StringManager smClient) {
         writer.println(smClient.getString("managerServlet.sslConnectorTrustedCerts"));
-        Map<String,List<String>> connectorTrustedCerts = getConnectorTrustedCerts(smClient);
+        Map<String,List<String>> connectorTrustedCerts = getConnectorTrustedCerts();
         for (Map.Entry<String,List<String>> entry : connectorTrustedCerts.entrySet()) {
             writer.println(entry.getKey());
             for (String cert : entry.getValue()) {
@@ -1132,31 +1137,21 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
             writer.println(smClient.getString("managerServlet.resourcesAll"));
         }
 
-        printResources(writer, "", global, type, smClient);
+        Class<?> clazz = null;
+        try {
+            if (type != null) {
+                clazz = Class.forName(type);
+            }
+        } catch (Throwable t) {
+            ExceptionUtils.handleThrowable(t);
+            log("ManagerServlet.resources[" + type + "]", t);
+            writer.println(smClient.getString("managerServlet.exception",
+                    t.toString()));
+            return;
+        }
 
-    }
+        printResources(writer, "", global, type, clazz, smClient);
 
-
-    /**
-     * List the resources of the given context.
-     *
-     * @param writer Writer to render to
-     * @param prefix Path for recursion
-     * @param namingContext The naming context for lookups
-     * @param type Fully qualified class name of the resource type of interest,
-     *  or <code>null</code> to list resources of all types
-     * @param clazz Unused
-     * @param smClient i18n support for current client's locale
-     *
-     * @deprecated Use {@link #printResources(PrintWriter, String,
-     *             javax.naming.Context, String, StringManager)}
-     *             This method will be removed in Tomcat 10.x onwards
-     */
-    @Deprecated
-    protected void printResources(PrintWriter writer, String prefix,
-            javax.naming.Context namingContext,
-            String type, Class<?> clazz, StringManager smClient) {
-        printResources(writer, prefix, namingContext, type, smClient);
     }
 
 
@@ -1167,23 +1162,27 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
      * @param namingContext The naming context for lookups
      * @param type Fully qualified class name of the resource type of interest,
      *  or <code>null</code> to list resources of all types
+     * @param clazz The resource class or <code>null</code> to list
+     *  resources of all types
      * @param smClient i18n support for current client's locale
      */
     protected void printResources(PrintWriter writer, String prefix,
                                   javax.naming.Context namingContext,
-                                  String type,
+                                  String type, Class<?> clazz,
                                   StringManager smClient) {
+
         try {
             NamingEnumeration<Binding> items = namingContext.listBindings("");
             while (items.hasMore()) {
                 Binding item = items.next();
-                Object obj = item.getObject();
-                if (obj instanceof javax.naming.Context) {
-                    printResources(writer, prefix + item.getName() + "/",
-                            (javax.naming.Context) obj, type, smClient);
+                if (item.getObject() instanceof javax.naming.Context) {
+                    printResources
+                        (writer, prefix + item.getName() + "/",
+                         (javax.naming.Context) item.getObject(), type, clazz,
+                         smClient);
                 } else {
-                    if (type != null && (obj == null ||
-                            !IntrospectionUtils.isInstance(obj.getClass(), type))) {
+                    if ((clazz != null) &&
+                        (!(clazz.isInstance(item.getObject())))) {
                         continue;
                     }
                     writer.print(prefix + item.getName());
@@ -1199,6 +1198,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
             writer.println(smClient.getString("managerServlet.exception",
                     t.toString()));
         }
+
     }
 
 
@@ -1211,9 +1211,21 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
         if (debug >= 1)
             log("serverinfo");
         try {
-            writer.println(smClient.getString("managerServlet.serverInfo", ServerInfo.getServerInfo(),
-                    System.getProperty("os.name"), System.getProperty("os.version"), System.getProperty("os.arch"),
-                    System.getProperty("java.runtime.version"), System.getProperty("java.vm.vendor")));
+            StringBuilder props = new StringBuilder();
+            props.append("OK - Server info");
+            props.append("\nTomcat Version: ");
+            props.append(ServerInfo.getServerInfo());
+            props.append("\nOS Name: ");
+            props.append(System.getProperty("os.name"));
+            props.append("\nOS Version: ");
+            props.append(System.getProperty("os.version"));
+            props.append("\nOS Architecture: ");
+            props.append(System.getProperty("os.arch"));
+            props.append("\nJVM Version: ");
+            props.append(System.getProperty("java.runtime.version"));
+            props.append("\nJVM Vendor: ");
+            props.append(System.getProperty("java.vm.vendor"));
+            writer.println(props.toString());
         } catch (Throwable t) {
             ExceptionUtils.handleThrowable(t);
             getServletContext().log("ManagerServlet.serverinfo",t);
@@ -1677,7 +1689,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
 
 
     protected static boolean validateContextName(ContextName cn,
-            PrintWriter writer, StringManager smClient) {
+            PrintWriter writer, StringManager sm) {
 
         // ContextName should be non-null with a path that is empty or starts
         // with /
@@ -1690,7 +1702,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
         if (cn != null) {
             path = Escape.htmlElementContent(cn.getPath());
         }
-        writer.println(smClient.getString("managerServlet.invalidPath", path));
+        writer.println(sm.getString("managerServlet.invalidPath", path));
         return false;
     }
 
@@ -1763,7 +1775,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
     }
 
 
-    protected Map<String,List<String>> getConnectorCiphers(StringManager smClient) {
+    protected Map<String,List<String>> getConnectorCiphers() {
         Map<String,List<String>> result = new HashMap<>();
 
         Connector connectors[] = getConnectors();
@@ -1778,7 +1790,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                 }
             } else {
                 ArrayList<String> cipherList = new ArrayList<>(1);
-                cipherList.add(smClient.getString("managerServlet.notSslConnector"));
+                cipherList.add(sm.getString("managerServlet.notSslConnector"));
                 result.put(connector.toString(), cipherList);
             }
         }
@@ -1786,7 +1798,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
     }
 
 
-    protected Map<String,List<String>> getConnectorCerts(StringManager smClient) {
+    protected Map<String,List<String>> getConnectorCerts() {
         Map<String,List<String>> result = new HashMap<>();
 
         Connector connectors[] = getConnectors();
@@ -1809,7 +1821,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                             }
                             X509Certificate[] certs = sslContext.getCertificateChain(alias);
                             if (certs == null) {
-                                certList.add(smClient.getString("managerServlet.certsNotAvailable"));
+                                certList.add(sm.getString("managerServlet.certsNotAvailable"));
                             } else {
                                 for (Certificate cert : certs) {
                                     certList.add(cert.toString());
@@ -1819,14 +1831,14 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                         }
                     } else {
                         List<String> certList = new ArrayList<>();
-                        certList.add(smClient.getString("managerServlet.certsNotAvailable"));
+                        certList.add(sm.getString("managerServlet.certsNotAvailable"));
                         String name = connector.toString() + "-" + sslHostConfig.getHostName();
                         result.put(name, certList);
                     }
                 }
             } else {
                 List<String> certList = new ArrayList<>(1);
-                certList.add(smClient.getString("managerServlet.notSslConnector"));
+                certList.add(sm.getString("managerServlet.notSslConnector"));
                 result.put(connector.toString(), certList);
             }
         }
@@ -1835,7 +1847,7 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
     }
 
 
-    protected Map<String,List<String>> getConnectorTrustedCerts(StringManager smClient) {
+    protected Map<String,List<String>> getConnectorTrustedCerts() {
         Map<String,List<String>> result = new HashMap<>();
 
         Connector connectors[] = getConnectors();
@@ -1851,22 +1863,22 @@ public class ManagerServlet extends HttpServlet implements ContainerServlet {
                                 sslHostConfig.getCertificates().iterator().next().getSslContext();
                         X509Certificate[] certs = sslContext.getAcceptedIssuers();
                         if (certs == null) {
-                            certList.add(smClient.getString("managerServlet.certsNotAvailable"));
+                            certList.add(sm.getString("managerServlet.certsNotAvailable"));
                         } else if (certs.length == 0) {
-                            certList.add(smClient.getString("managerServlet.trustedCertsNotConfigured"));
+                            certList.add(sm.getString("managerServlet.trustedCertsNotConfigured"));
                         } else {
                             for (Certificate cert : certs) {
                                 certList.add(cert.toString());
                             }
                         }
                     } else {
-                        certList.add(smClient.getString("managerServlet.certsNotAvailable"));
+                        certList.add(sm.getString("managerServlet.certsNotAvailable"));
                     }
                     result.put(name, certList);
                 }
             } else {
                 List<String> certList = new ArrayList<>(1);
-                certList.add(smClient.getString("managerServlet.notSslConnector"));
+                certList.add(sm.getString("managerServlet.notSslConnector"));
                 result.put(connector.toString(), certList);
             }
         }
