@@ -138,14 +138,6 @@ public class JAASRealm extends RealmBase {
     protected String appName = null;
 
 
-    /**
-     * Descriptive information about this <code>Realm</code> implementation.
-     * @deprecated This will be removed in Tomcat 9 onwards.
-     */
-    @Deprecated
-    protected static final String name = "JAASRealm";
-
-
     /**
      * The list of role class names, split out for easy processing.
      */
@@ -172,7 +164,7 @@ public class JAASRealm extends RealmBase {
      */
     protected String configFile;
 
-    protected Configuration jaasConfiguration;
+    protected volatile Configuration jaasConfiguration;
     protected volatile boolean jaasConfigurationLoaded = false;
 
 
@@ -248,7 +240,7 @@ public class JAASRealm extends RealmBase {
      protected String roleClassNames = null;
 
      public String getRoleClassNames() {
-         return (this.roleClassNames);
+         return this.roleClassNames;
      }
 
      /**
@@ -304,7 +296,7 @@ public class JAASRealm extends RealmBase {
      protected String userClassNames = null;
 
      public String getUserClassNames() {
-         return (this.userClassNames);
+         return this.userClassNames;
      }
 
      /**
@@ -402,7 +394,7 @@ public class JAASRealm extends RealmBase {
         } catch (Throwable e) {
             ExceptionUtils.handleThrowable(e);
             log.error(sm.getString("jaasRealm.unexpectedError"), e);
-            return (null);
+            return null;
         } finally {
             if(!isUseContextClassLoader()) {
               Thread.currentThread().setContextClassLoader(ocl);
@@ -420,27 +412,27 @@ public class JAASRealm extends RealmBase {
             if (subject == null) {
                 if( log.isDebugEnabled())
                     log.debug(sm.getString("jaasRealm.failedLogin", username));
-                return (null);
+                return null;
             }
         } catch (AccountExpiredException e) {
             if (log.isDebugEnabled())
                 log.debug(sm.getString("jaasRealm.accountExpired", username));
-            return (null);
+            return null;
         } catch (CredentialExpiredException e) {
             if (log.isDebugEnabled())
                 log.debug(sm.getString("jaasRealm.credentialExpired", username));
-            return (null);
+            return null;
         } catch (FailedLoginException e) {
             if (log.isDebugEnabled())
                 log.debug(sm.getString("jaasRealm.failedLogin", username));
-            return (null);
+            return null;
         } catch (LoginException e) {
             log.warn(sm.getString("jaasRealm.loginException", username), e);
-            return (null);
+            return null;
         } catch (Throwable e) {
             ExceptionUtils.handleThrowable(e);
             log.error(sm.getString("jaasRealm.unexpectedError"), e);
-            return (null);
+            return null;
         }
 
         if( log.isDebugEnabled())
@@ -450,13 +442,13 @@ public class JAASRealm extends RealmBase {
         Principal principal = createPrincipal(username, subject, loginContext);
         if (principal == null) {
             log.debug(sm.getString("jaasRealm.authenticateFailure", username));
-            return (null);
+            return null;
         }
         if (log.isDebugEnabled()) {
-            log.debug(sm.getString("jaasRealm.authenticateSuccess", username));
+            log.debug(sm.getString("jaasRealm.authenticateSuccess", username, principal));
         }
 
-        return (principal);
+        return principal;
         } catch( Throwable t) {
             log.error( "error ", t);
             return null;
@@ -464,13 +456,6 @@ public class JAASRealm extends RealmBase {
     }
 
 
-    @Override
-    @Deprecated
-    protected String getName() {
-        return name;
-    }
-
-
     /**
      * @return the password associated with the given principal's user name. This
      * always returns null as the JAASRealm has no way of obtaining this
@@ -478,9 +463,7 @@ public class JAASRealm extends RealmBase {
      */
     @Override
     protected String getPassword(String username) {
-
-        return (null);
-
+        return null;
     }
 
 
@@ -618,6 +601,8 @@ public class JAASRealm extends RealmBase {
      * @return the loaded configuration
      */
     protected Configuration getConfig() {
+        // Local copy to avoid possible NPE due to concurrent change
+        String configFile = this.configFile;
         try {
             if (jaasConfigurationLoaded) {
                 return jaasConfiguration;
@@ -627,8 +612,7 @@ public class JAASRealm extends RealmBase {
                     jaasConfigurationLoaded = true;
                     return null;
                 }
-                URL resource = Thread.currentThread().getContextClassLoader().
-                        getResource(configFile);
+                URL resource = Thread.currentThread().getContextClassLoader().getResource(configFile);
                 URI uri = resource.toURI();
                 @SuppressWarnings("unchecked")
                 Class<Configuration> sunConfigFile = (Class<Configuration>)
@@ -640,23 +624,11 @@ public class JAASRealm extends RealmBase {
                 this.jaasConfigurationLoaded = true;
                 return this.jaasConfiguration;
             }
-        } catch (URISyntaxException ex) {
-            throw new RuntimeException(ex);
-        } catch (NoSuchMethodException ex) {
-            throw new RuntimeException(ex);
-        } catch (SecurityException ex) {
-            throw new RuntimeException(ex);
-        } catch (InstantiationException ex) {
-            throw new RuntimeException(ex);
-        } catch (IllegalAccessException ex) {
-            throw new RuntimeException(ex);
-        } catch (IllegalArgumentException ex) {
-            throw new RuntimeException(ex);
         } catch (InvocationTargetException ex) {
             throw new RuntimeException(ex.getCause());
-        } catch (ClassNotFoundException ex) {
+        } catch (SecurityException | URISyntaxException | ReflectiveOperationException |
+                IllegalArgumentException ex) {
             throw new RuntimeException(ex);
         }
-
     }
 }
