@@ -21,6 +21,7 @@ import java.io.InputStream;
 import java.security.Principal;
 import java.util.ArrayList;
 import java.util.HashMap;
+import java.util.List;
 import java.util.Map;
 
 import org.apache.catalina.LifecycleException;
@@ -55,14 +56,6 @@ public class MemoryRealm  extends RealmBase {
     private static Digester digester = null;
 
 
-    /**
-     * Descriptive information about this Realm implementation.
-     * @deprecated This will be removed in Tomcat 9 onwards.
-     */
-    @Deprecated
-    protected static final String name = "MemoryRealm";
-
-
     /**
      * The pathname (absolute or relative to Catalina's current working
      * directory) of the XML file containing our database information.
@@ -163,7 +156,7 @@ public class MemoryRealm  extends RealmBase {
     void addUser(String username, String password, String roles) {
 
         // Accumulate the list of roles for this user
-        ArrayList<String> list = new ArrayList<>();
+        List<String> list = new ArrayList<>();
         roles += ",";
         while (true) {
             int comma = roles.indexOf(',');
@@ -190,7 +183,6 @@ public class MemoryRealm  extends RealmBase {
      * the XML input file, creating a new one if necessary.
      */
     protected synchronized Digester getDigester() {
-
         if (digester == null) {
             digester = new Digester();
             digester.setValidating(false);
@@ -203,15 +195,7 @@ public class MemoryRealm  extends RealmBase {
             }
             digester.addRuleSet(new MemoryRuleSet());
         }
-        return (digester);
-
-    }
-
-
-    @Override
-    @Deprecated
-    protected String getName() {
-        return name;
+        return digester;
     }
 
 
@@ -223,9 +207,9 @@ public class MemoryRealm  extends RealmBase {
 
         GenericPrincipal principal = principals.get(username);
         if (principal != null) {
-            return (principal.getPassword());
+            return principal.getPassword();
         } else {
-            return (null);
+            return null;
         }
 
     }
