@@ -577,6 +577,11 @@ public class HostConfig implements LifecycleListener {
                 }
             }
 
+            if (context.getPath() != null) {
+                log.warn(sm.getString("hostConfig.deployDescriptor.path", context.getPath(),
+                        contextXml.getAbsolutePath()));
+            }
+
             Class<?> clazz = Class.forName(host.getConfigClass());
             LifecycleListener listener = (LifecycleListener) clazz.getConstructor().newInstance();
             context.addLifecycleListener(listener);
@@ -603,6 +608,17 @@ public class HostConfig implements LifecycleListener {
                     if (docBase.getAbsolutePath().toLowerCase(Locale.ENGLISH).endsWith(".war")) {
                         isExternalWar = true;
                     }
+                    // Check that a WAR or DIR in the appBase is not 'hidden'
+                    File war = new File(host.getAppBaseFile(), cn.getBaseName() + ".war");
+                    if (war.exists()) {
+                        log.warn(sm.getString("hostConfig.deployDescriptor.hiddenWar",
+                                contextXml.getAbsolutePath(), war.getAbsolutePath()));
+                    }
+                    File dir = new File(host.getAppBaseFile(), cn.getBaseName());
+                    if (dir.exists()) {
+                        log.warn(sm.getString("hostConfig.deployDescriptor.hiddenDir",
+                                contextXml.getAbsolutePath(), dir.getAbsolutePath()));
+                    }
                 } else {
                     log.warn(sm.getString("hostConfig.deployDescriptor.localDocBaseSpecified",
                              docBase));
@@ -1403,7 +1419,8 @@ public class HostConfig implements LifecycleListener {
             try {
                 context.start();
             } catch (Exception e) {
-                log.error(sm.getString("hostConfig.context.restart", app.name), e);
+                log.warn(sm.getString
+                         ("hostConfig.context.restart", app.name), e);
             }
         }
     }
@@ -1554,7 +1571,7 @@ public class HostConfig implements LifecycleListener {
             Registry.getRegistry(null, null).registerComponent
                 (this, oname, this.getClass().getName());
         } catch (Exception e) {
-            log.warn(sm.getString("hostConfig.jmx.register", oname), e);
+            log.error(sm.getString("hostConfig.jmx.register", oname), e);
         }
 
         if (!host.getAppBaseFile().isDirectory()) {
@@ -1582,7 +1599,7 @@ public class HostConfig implements LifecycleListener {
             try {
                 Registry.getRegistry(null, null).unregisterComponent(oname);
             } catch (Exception e) {
-                log.warn(sm.getString("hostConfig.jmx.unregister", oname), e);
+                log.error(sm.getString("hostConfig.jmx.unregister", oname), e);
             }
         }
         oname = null;
