@@ -17,20 +17,14 @@
 package org.apache.catalina.startup;
 
 import java.io.File;
-import java.io.FileOutputStream;
 import java.io.IOException;
-import java.io.InputStream;
-import java.io.OutputStream;
-import java.lang.reflect.InvocationTargetException;
 import java.net.MalformedURLException;
 import java.net.URL;
-import java.net.URLConnection;
 import java.security.Principal;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
-import java.util.Properties;
 import java.util.Stack;
 import java.util.jar.JarEntry;
 import java.util.jar.JarFile;
@@ -66,8 +60,6 @@ import org.apache.catalina.core.StandardService;
 import org.apache.catalina.core.StandardWrapper;
 import org.apache.catalina.realm.GenericPrincipal;
 import org.apache.catalina.realm.RealmBase;
-import org.apache.catalina.util.ContextName;
-import org.apache.catalina.util.IOTools;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.UriUtil;
 import org.apache.tomcat.util.descriptor.web.LoginConfig;
@@ -112,17 +104,12 @@ import org.apache.tomcat.util.res.StringManager;
  * this class.
  *
  * <p>
- * This class provides a set of convenience methods for configuring web
- * application contexts; all overloads of the method <code>addWebapp()</code>.
- * These methods are equivalent to adding a web application to the Host's
- * appBase (normally the webapps directory). These methods create a Context,
- * configure it with the equivalent of the defaults provided by
- * <code>conf/web.xml</code> (see {@link #initWebappDefaults(String)} for
- * details) and add the Context to a Host. These methods do not use a global
- * default web.xml; rather, they add a {@link LifecycleListener} to configure
- * the defaults. Any WEB-INF/web.xml and META-INF/context.xml packaged with the
- * application will be processed normally. Normal web fragment and
- * {@link javax.servlet.ServletContainerInitializer} processing will be applied.
+ * This class provides a set of convenience methods for configuring webapp
+ * contexts, all overloads of the method <code>addWebapp</code>. These methods
+ * create a webapp context, configure it, and then add it to a {@link Host}.
+ * They do not use a global default web.xml; rather, they add a lifecycle
+ * listener that adds the standard DefaultServlet, JSP processing, and welcome
+ * files.
  *
  * <p>
  * In complex cases, you may prefer to use the ordinary Tomcat API to create
@@ -166,7 +153,6 @@ public class Tomcat {
     protected int port = 8080;
     protected String hostname = "localhost";
     protected String basedir;
-    protected boolean defaultConnectorCreated = false;
 
     private final Map<String, String> userPass = new HashMap<>();
     private final Map<String, List<String>> userRoles = new HashMap<>();
@@ -179,30 +165,34 @@ public class Tomcat {
     }
 
     /**
-     * Tomcat needs a directory for temp files. This should be the
-     * first method called.
-     *
+     * Tomcat requires that the base directory is set because the defaults for
+     * a number of other locations, such as the work directory, are derived from
+     * the base directory. This should be the first method called.
      * <p>
-     * By default, if this method is not called, we use:
-     * <ul>
-     *  <li>system properties - catalina.base, catalina.home</li>
-     *  <li>[user.dir]/tomcat.$PORT</li>
-     * </ul>
-     * (/tmp doesn't seem a good choice for security).
-     *
+     * If this method is not called then Tomcat will attempt to use these
+     * locations in the following order:
+     * <ol>
+     *  <li>if set, the catalina.base system property</li>
+     *  <li>if set, the catalina.home system property</li>
+     *  <li>The user.dir system property (the directory where Java was run from)
+     *      where a directory named tomcat.$PORT will be created. $PORT is the
+     *      value configured via {@link #setPort(int)} which defaults to 8080 if
+     *      not set</li>
+     * </ol>
+     * The user should ensure that the file permissions for the base directory
+     * are appropriate.
      * <p>
      * TODO: disable work dir if not needed ( no jsp, etc ).
      *
-     * @param basedir The Tomcat base folder on which all others
-     *  will be derived
+     * @param basedir The Tomcat base folder on which all others will be derived
      */
     public void setBaseDir(String basedir) {
         this.basedir = basedir;
     }
 
     /**
-     * Set the port for the default connector. Must
-     * be called before start().
+     * Set the port for the default connector. The default connector will
+     * only be created if getConnector is called.
      * @param port The port number
      */
     public void setPort(int port) {
@@ -218,22 +208,17 @@ public class Tomcat {
         hostname = s;
     }
 
-
     /**
-     * This is equivalent to adding a web application to a Host's appBase
-     * (usually Tomcat's webapps directory). By default, the equivalent of the
-     * default web.xml will be applied to the web application (see
-     * {@link #initWebappDefaults(String)}). This may be prevented by calling
-     * {@link #setAddDefaultWebXmlToWebapp(boolean)} with {@code false}. Any
-     * <code>WEB-INF/web.xml</code> and <code>META-INF/context.xml</code>
-     * packaged with the application will always be processed and normal web
-     * fragment and {@link javax.servlet.ServletContainerInitializer} processing
-     * will always be applied.
+     * This is equivalent to adding a web application to Tomcat's webapps
+     * directory. The equivalent of the default web.xml will be applied  to the
+     * web application and any WEB-INF/web.xml and META-INF/context.xml packaged
+     * with the application will be processed normally. Normal web fragment and
+     * {@link javax.servlet.ServletContainerInitializer} processing will be
+     * applied.
      *
      * @param contextPath The context mapping to use, "" for root context.
-     * @param docBase     Base directory for the context, for static files. Must
-     *                        exist, relative to the server home
-     *
+     * @param docBase Base directory for the context, for static files.
+     *  Must exist, relative to the server home
      * @return the deployed context
      */
     public Context addWebapp(String contextPath, String docBase) {
@@ -241,57 +226,6 @@ public class Tomcat {
     }
 
 
-    /**
-     * Copy the specified WAR file to the Host's appBase and then call
-     * {@link #addWebapp(String, String)} with the newly copied WAR. The WAR
-     * will <b>NOT</b> be removed from the Host's appBase when the Tomcat
-     * instance stops. Note that {@link ExpandWar} provides utility methods that
-     * may be used to delete the WAR and/or expanded directory if required.
-     *
-     * @param contextPath   The context mapping to use, "" for root context.
-     * @param source        The location from which the WAR should be copied
-     *
-     * @return The deployed Context
-     *
-     * @throws IOException If an I/O error occurs while copying the WAR file
-     *                     from the specified URL to the appBase
-     */
-    public Context addWebapp(String contextPath, URL source) throws IOException {
-
-        ContextName cn = new ContextName(contextPath, null);
-
-        // Make sure a conflicting web application has not already been deployed
-        Host h = getHost();
-        if (h.findChild(cn.getName()) != null) {
-            throw new IllegalArgumentException(sm.getString("tomcat.addWebapp.conflictChild",
-                    source, contextPath, cn.getName()));
-        }
-
-        // Make sure appBase does not contain a conflicting web application
-        File targetWar = new File(h.getAppBaseFile(), cn.getBaseName() + ".war");
-        File targetDir = new File(h.getAppBaseFile(), cn.getBaseName());
-
-        if (targetWar.exists()) {
-            throw new IllegalArgumentException(sm.getString("tomcat.addWebapp.conflictFile",
-                    source, contextPath, targetWar.getAbsolutePath()));
-        }
-        if (targetDir.exists()) {
-            throw new IllegalArgumentException(sm.getString("tomcat.addWebapp.conflictFile",
-                    source, contextPath, targetDir.getAbsolutePath()));
-        }
-
-        // Should be good to copy the WAR now
-        URLConnection uConn = source.openConnection();
-
-        try (InputStream is = uConn.getInputStream();
-                OutputStream os = new FileOutputStream(targetWar)) {
-            IOTools.flow(is, os);
-        }
-
-        return addWebapp(contextPath, targetWar.getAbsolutePath());
-    }
-
-
     /**
      * Add a context - programmatic mode, no default web.xml used. This means
      * that there is no JSP support (no JSP servlet), no default servlet and
@@ -424,7 +358,6 @@ public class Tomcat {
      */
     public void init() throws LifecycleException {
         getServer();
-        getConnector();
         server.init();
     }
 
@@ -436,7 +369,6 @@ public class Tomcat {
      */
     public void start() throws LifecycleException {
         getServer();
-        getConnector();
         server.start();
     }
 
@@ -492,24 +424,19 @@ public class Tomcat {
     // You can tune individual Tomcat objects, using internal APIs
 
     /**
-     * Get the default http connector. You can set more
-     * parameters - the port is already initialized.
+     * Get the default HTTP connector that is used by the embedded
+     * Tomcat. It is first configured connector in the service.
+     * If there's no connector defined, it will create and add a default
+     * connector using the port and address specified in this Tomcat
+     * instance, and return it for further customization.
      *
-     * <p>
-     * Alternatively, you can construct a Connector and set any params,
-     * then call addConnector(Connector)
-     *
-     * @return A connector object that can be customized
+     * @return The connector object
      */
     public Connector getConnector() {
         Service service = getService();
         if (service.findConnectors().length > 0) {
             return service.findConnectors()[0];
         }
-
-        if (defaultConnectorCreated) {
-            return null;
-        }
         // The same as in standard Tomcat configuration.
         // This creates an APR HTTP connector if AprLifecycleListener has been
         // configured (created) and Tomcat Native library is available.
@@ -517,12 +444,15 @@ public class Tomcat {
         Connector connector = new Connector("HTTP/1.1");
         connector.setPort(port);
         service.addConnector(connector);
-        defaultConnectorCreated = true;
         return connector;
     }
 
+    /**
+     * Set the specified connector in the service, if it is not already
+     * present.
+     * @param connector The connector instance to add
+     */
     public void setConnector(Connector connector) {
-        defaultConnectorCreated = true;
         Service service = getService();
         boolean found = false;
         for (Connector serviceConnector : service.findConnectors()) {
@@ -656,24 +586,13 @@ public class Tomcat {
         return ctx;
     }
 
-
     /**
-     * This is equivalent to adding a web application to a Host's appBase
-     * (usually Tomcat's webapps directory). By default, the equivalent of the
-     * default web.xml will be applied to the web application (see
-     * {@link #initWebappDefaults(String)}). This may be prevented by calling
-     * {@link #setAddDefaultWebXmlToWebapp(boolean)} with {@code false}. Any
-     * <code>WEB-INF/web.xml</code> and <code>META-INF/context.xml</code>
-     * packaged with the application will always be processed and normal web
-     * fragment and {@link javax.servlet.ServletContainerInitializer} processing
-     * will always be applied.
-     *
-     * @param host        The host in which the context will be deployed
+     * @param host The host in which the context will be deployed
      * @param contextPath The context mapping to use, "" for root context.
-     * @param docBase     Base directory for the context, for static files. Must
-     *                        exist, relative to the server home
-     *
+     * @param docBase Base directory for the context, for static files.
+     *  Must exist, relative to the server home
      * @return the deployed context
+     * @see #addWebapp(String, String)
      */
     public Context addWebapp(Host host, String contextPath, String docBase) {
         LifecycleListener listener = null;
@@ -686,10 +605,9 @@ public class Tomcat {
             throw new IllegalArgumentException(e);
         }
 
-        return addWebapp(host, contextPath, docBase, listener);
+        return addWebapp(host,  contextPath, docBase, listener);
     }
 
-
     /**
      * @param host The host in which the context will be deployed
      * @param contextPath The context mapping to use, "" for root context.
@@ -698,36 +616,6 @@ public class Tomcat {
      * @param config Custom context configurator helper
      * @return the deployed context
      * @see #addWebapp(String, String)
-     *
-     * @deprecated Use {@link
-     *             #addWebapp(Host, String, String, LifecycleListener)} instead
-     */
-    @Deprecated
-    public Context addWebapp(Host host, String contextPath, String docBase, ContextConfig config) {
-        return addWebapp(host, contextPath, docBase, (LifecycleListener) config);
-    }
-
-
-    /**
-     * This is equivalent to adding a web application to a Host's appBase
-     * (usually Tomcat's webapps directory). By default, the equivalent of the
-     * default web.xml will be applied to the web application (see
-     * {@link #initWebappDefaults(String)}). This may be prevented by calling
-     * {@link #setAddDefaultWebXmlToWebapp(boolean)} with {@code false}. Any
-     * <code>WEB-INF/web.xml</code> and <code>META-INF/context.xml</code>
-     * packaged with the application will always be processed and normal web
-     * fragment and {@link javax.servlet.ServletContainerInitializer} processing
-     * will always be applied.
-     *
-     * @param host        The host in which the context will be deployed
-     * @param contextPath The context mapping to use, "" for root context.
-     * @param docBase     Base directory for the context, for static files. Must
-     *                        exist, relative to the server home
-     * @param config      Custom context configuration helper. Any configuration
-     *                        will be in addition to equivalent of the default
-     *                        web.xml configuration described above.
-     *
-     * @return the deployed context
      */
     public Context addWebapp(Host host, String contextPath, String docBase,
             LifecycleListener config) {
@@ -738,9 +626,8 @@ public class Tomcat {
         ctx.setPath(contextPath);
         ctx.setDocBase(docBase);
 
-        if (addDefaultWebXmlToWebapp) {
+        if (addDefaultWebXmlToWebapp)
             ctx.addLifecycleListener(getDefaultWebXmlListener());
-        }
 
         ctx.setConfigFile(getWebappConfigFile(docBase, contextPath));
 
@@ -791,33 +678,30 @@ public class Tomcat {
      * @return a realm instance
      */
     protected Realm createDefaultRealm() {
-        return new RealmBase() {
-            @Override
-            @Deprecated
-            protected String getName() {
-                return "Simple";
-            }
+        return new SimpleRealm();
+    }
 
-            @Override
-            protected String getPassword(String username) {
-                return userPass.get(username);
-            }
 
-            @Override
-            protected Principal getPrincipal(String username) {
-                Principal p = userPrincipals.get(username);
-                if (p == null) {
-                    String pass = userPass.get(username);
-                    if (pass != null) {
-                        p = new GenericPrincipal(username, pass,
-                                userRoles.get(username));
-                        userPrincipals.put(username, p);
-                    }
+    private class SimpleRealm extends RealmBase {
+
+        @Override
+        protected String getPassword(String username) {
+            return userPass.get(username);
+        }
+
+        @Override
+        protected Principal getPrincipal(String username) {
+            Principal p = userPrincipals.get(username);
+            if (p == null) {
+                String pass = userPass.get(username);
+                if (pass != null) {
+                    p = new GenericPrincipal(username, pass,
+                            userRoles.get(username));
+                    userPrincipals.put(username, p);
                 }
-                return p;
             }
-
-        };
+            return p;
+        }
     }
 
 
@@ -993,10 +877,7 @@ public class Tomcat {
         try {
             return (Context) Class.forName(contextClass).getConstructor()
                     .newInstance();
-        } catch (InstantiationException | IllegalAccessException
-                | IllegalArgumentException | InvocationTargetException
-                | NoSuchMethodException | SecurityException
-                | ClassNotFoundException e) {
+        } catch (ReflectiveOperationException  | IllegalArgumentException | SecurityException e) {
             throw new IllegalArgumentException(
                     "Can't instantiate context-class " + contextClass
                             + " for host " + host + " and url "
@@ -1039,32 +920,22 @@ public class Tomcat {
         }
     }
 
-
     /**
-     * Provide default configuration for a context. This is broadly the
-     * programmatic equivalent of the default web.xml and provides the following
-     * features:
-     * <ul>
-     * <li>Default servlet mapped to "/"</li>
-     * <li>JSP servlet mapped to "*.jsp" and ""*.jspx"</li>
-     * <li>Session timeout of 30 minutes</li>
-     * <li>MIME mappings (subset of those in conf/web.xml)</li>
-     * <li>Welcome files</li>
-     * </ul>
-     * TODO: Align the MIME mappings with conf/web.xml - possibly via a common
-     *       file.
+     * Provide default configuration for a context. This is the programmatic
+     * equivalent of the default web.xml.
      *
-     * @param contextPath   The path of the context to set the defaults for
+     *  TODO: in normal Tomcat, if default-web.xml is not found, use this
+     *  method
+     *
+     * @param contextPath   The context to set the defaults for
      */
     public void initWebappDefaults(String contextPath) {
         Container ctx = getHost().findChild(contextPath);
         initWebappDefaults((Context) ctx);
     }
 
-
     /**
-     * Static version of {@link #initWebappDefaults(String)}.
-     *
+     * Static version of {@link #initWebappDefaults(String)}
      * @param ctx   The context to set the defaults for
      */
     public static void initWebappDefaults(Context ctx) {
@@ -1089,8 +960,11 @@ public class Tomcat {
         // Sessions
         ctx.setSessionTimeout(30);
 
-        // MIME type mappings
-        addDefaultMimeTypeMappings(ctx);
+        // MIME mappings
+        for (int i = 0; i < DEFAULT_MIME_MAPPINGS.length;) {
+            ctx.addMimeMapping(DEFAULT_MIME_MAPPINGS[i++],
+                    DEFAULT_MIME_MAPPINGS[i++]);
+        }
 
         // Welcome files
         ctx.addWelcomeFile("index.html");
@@ -1099,25 +973,6 @@ public class Tomcat {
     }
 
 
-    /**
-     * Add the default MIME type mappings to the provide Context.
-     *
-     * @param context The web application to which the default MIME type
-     *                mappings should be added.
-     */
-    public static void addDefaultMimeTypeMappings(Context context) {
-        Properties defaultMimeMappings = new Properties();
-        try (InputStream is = Tomcat.class.getResourceAsStream("MimeTypeMappings.properties")) {
-            defaultMimeMappings.load(is);
-            for (Map.Entry<Object, Object>  entry: defaultMimeMappings.entrySet()) {
-                context.addMimeMapping((String) entry.getKey(), (String) entry.getValue());
-            }
-        } catch (IOException e) {
-            throw new IllegalStateException(sm.getString("tomcat.defaultMimeTypeMappingsFail"), e);
-        }
-    }
-
-
     /**
      * Fix startup sequence - required if you don't use web.xml.
      *
@@ -1230,12 +1085,191 @@ public class Tomcat {
         }
     }
 
+    /**
+     * TODO: would a properties resource be better ? Or just parsing
+     * /etc/mime.types ?
+     * This is needed because we don't use the default web.xml, where this
+     * is encoded.
+     */
+    private static final String[] DEFAULT_MIME_MAPPINGS = {
+        "abs", "audio/x-mpeg",
+        "ai", "application/postscript",
+        "aif", "audio/x-aiff",
+        "aifc", "audio/x-aiff",
+        "aiff", "audio/x-aiff",
+        "aim", "application/x-aim",
+        "art", "image/x-jg",
+        "asf", "video/x-ms-asf",
+        "asx", "video/x-ms-asf",
+        "au", "audio/basic",
+        "avi", "video/x-msvideo",
+        "avx", "video/x-rad-screenplay",
+        "bcpio", "application/x-bcpio",
+        "bin", "application/octet-stream",
+        "bmp", "image/bmp",
+        "body", "text/html",
+        "cdf", "application/x-cdf",
+        "cer", "application/pkix-cert",
+        "class", "application/java",
+        "cpio", "application/x-cpio",
+        "csh", "application/x-csh",
+        "css", "text/css",
+        "dib", "image/bmp",
+        "doc", "application/msword",
+        "dtd", "application/xml-dtd",
+        "dv", "video/x-dv",
+        "dvi", "application/x-dvi",
+        "eps", "application/postscript",
+        "etx", "text/x-setext",
+        "exe", "application/octet-stream",
+        "gif", "image/gif",
+        "gtar", "application/x-gtar",
+        "gz", "application/x-gzip",
+        "hdf", "application/x-hdf",
+        "hqx", "application/mac-binhex40",
+        "htc", "text/x-component",
+        "htm", "text/html",
+        "html", "text/html",
+        "ief", "image/ief",
+        "jad", "text/vnd.sun.j2me.app-descriptor",
+        "jar", "application/java-archive",
+        "java", "text/x-java-source",
+        "jnlp", "application/x-java-jnlp-file",
+        "jpe", "image/jpeg",
+        "jpeg", "image/jpeg",
+        "jpg", "image/jpeg",
+        "js", "application/javascript",
+        "jsf", "text/plain",
+        "jspf", "text/plain",
+        "kar", "audio/midi",
+        "latex", "application/x-latex",
+        "m3u", "audio/x-mpegurl",
+        "mac", "image/x-macpaint",
+        "man", "text/troff",
+        "mathml", "application/mathml+xml",
+        "me", "text/troff",
+        "mid", "audio/midi",
+        "midi", "audio/midi",
+        "mif", "application/x-mif",
+        "mov", "video/quicktime",
+        "movie", "video/x-sgi-movie",
+        "mp1", "audio/mpeg",
+        "mp2", "audio/mpeg",
+        "mp3", "audio/mpeg",
+        "mp4", "video/mp4",
+        "mpa", "audio/mpeg",
+        "mpe", "video/mpeg",
+        "mpeg", "video/mpeg",
+        "mpega", "audio/x-mpeg",
+        "mpg", "video/mpeg",
+        "mpv2", "video/mpeg2",
+        "nc", "application/x-netcdf",
+        "oda", "application/oda",
+        "odb", "application/vnd.oasis.opendocument.database",
+        "odc", "application/vnd.oasis.opendocument.chart",
+        "odf", "application/vnd.oasis.opendocument.formula",
+        "odg", "application/vnd.oasis.opendocument.graphics",
+        "odi", "application/vnd.oasis.opendocument.image",
+        "odm", "application/vnd.oasis.opendocument.text-master",
+        "odp", "application/vnd.oasis.opendocument.presentation",
+        "ods", "application/vnd.oasis.opendocument.spreadsheet",
+        "odt", "application/vnd.oasis.opendocument.text",
+        "otg", "application/vnd.oasis.opendocument.graphics-template",
+        "oth", "application/vnd.oasis.opendocument.text-web",
+        "otp", "application/vnd.oasis.opendocument.presentation-template",
+        "ots", "application/vnd.oasis.opendocument.spreadsheet-template ",
+        "ott", "application/vnd.oasis.opendocument.text-template",
+        "ogx", "application/ogg",
+        "ogv", "video/ogg",
+        "oga", "audio/ogg",
+        "ogg", "audio/ogg",
+        "spx", "audio/ogg",
+        "flac", "audio/flac",
+        "anx", "application/annodex",
+        "axa", "audio/annodex",
+        "axv", "video/annodex",
+        "xspf", "application/xspf+xml",
+        "pbm", "image/x-portable-bitmap",
+        "pct", "image/pict",
+        "pdf", "application/pdf",
+        "pgm", "image/x-portable-graymap",
+        "pic", "image/pict",
+        "pict", "image/pict",
+        "pls", "audio/x-scpls",
+        "png", "image/png",
+        "pnm", "image/x-portable-anymap",
+        "pnt", "image/x-macpaint",
+        "ppm", "image/x-portable-pixmap",
+        "ppt", "application/vnd.ms-powerpoint",
+        "pps", "application/vnd.ms-powerpoint",
+        "ps", "application/postscript",
+        "psd", "image/vnd.adobe.photoshop",
+        "qt", "video/quicktime",
+        "qti", "image/x-quicktime",
+        "qtif", "image/x-quicktime",
+        "ras", "image/x-cmu-raster",
+        "rdf", "application/rdf+xml",
+        "rgb", "image/x-rgb",
+        "rm", "application/vnd.rn-realmedia",
+        "roff", "text/troff",
+        "rtf", "application/rtf",
+        "rtx", "text/richtext",
+        "sh", "application/x-sh",
+        "shar", "application/x-shar",
+        /*"shtml", "text/x-server-parsed-html",*/
+        "sit", "application/x-stuffit",
+        "snd", "audio/basic",
+        "src", "application/x-wais-source",
+        "sv4cpio", "application/x-sv4cpio",
+        "sv4crc", "application/x-sv4crc",
+        "svg", "image/svg+xml",
+        "svgz", "image/svg+xml",
+        "swf", "application/x-shockwave-flash",
+        "t", "text/troff",
+        "tar", "application/x-tar",
+        "tcl", "application/x-tcl",
+        "tex", "application/x-tex",
+        "texi", "application/x-texinfo",
+        "texinfo", "application/x-texinfo",
+        "tif", "image/tiff",
+        "tiff", "image/tiff",
+        "tr", "text/troff",
+        "tsv", "text/tab-separated-values",
+        "txt", "text/plain",
+        "ulw", "audio/basic",
+        "ustar", "application/x-ustar",
+        "vxml", "application/voicexml+xml",
+        "xbm", "image/x-xbitmap",
+        "xht", "application/xhtml+xml",
+        "xhtml", "application/xhtml+xml",
+        "xls", "application/vnd.ms-excel",
+        "xml", "application/xml",
+        "xpm", "image/x-xpixmap",
+        "xsl", "application/xml",
+        "xslt", "application/xslt+xml",
+        "xul", "application/vnd.mozilla.xul+xml",
+        "xwd", "image/x-xwindowdump",
+        "vsd", "application/vnd.visio",
+        "wav", "audio/x-wav",
+        "wbmp", "image/vnd.wap.wbmp",
+        "wml", "text/vnd.wap.wml",
+        "wmlc", "application/vnd.wap.wmlc",
+        "wmls", "text/vnd.wap.wmlsc",
+        "wmlscriptc", "application/vnd.wap.wmlscriptc",
+        "wmv", "video/x-ms-wmv",
+        "wrl", "model/vrml",
+        "wspolicy", "application/wspolicy+xml",
+        "Z", "application/x-compress",
+        "z", "application/x-compress",
+        "zip", "application/zip"
+    };
+
     protected URL getWebappConfigFile(String path, String contextName) {
         File docBase = new File(path);
         if (docBase.isDirectory()) {
             return getWebappConfigFileFromDirectory(docBase, contextName);
         } else {
-            return getWebappConfigFileFromWar(docBase, contextName);
+            return getWebappConfigFileFromJar(docBase, contextName);
         }
     }
 
@@ -1253,7 +1287,7 @@ public class Tomcat {
         return result;
     }
 
-    private URL getWebappConfigFileFromWar(File docBase, String contextName) {
+    private URL getWebappConfigFileFromJar(File docBase, String contextName) {
         URL result = null;
         try (JarFile jar = new JarFile(docBase)) {
             JarEntry entry = jar.getJarEntry(Constants.ApplicationContextXml);
