@@ -17,7 +17,6 @@
 package org.apache.catalina.users;
 
 import java.io.File;
-import java.io.FileNotFoundException;
 import java.io.FileOutputStream;
 import java.io.IOException;
 import java.io.InputStream;
@@ -26,7 +25,6 @@ import java.io.PrintWriter;
 import java.net.URI;
 import java.net.URL;
 import java.net.URLConnection;
-import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.Map;
@@ -173,6 +171,7 @@ public class MemoryUserDatabase implements UserDatabase {
     }
 
 
+
     /**
      * @return the unique global identifier of this user database.
      */
@@ -416,6 +415,7 @@ public class MemoryUserDatabase implements UserDatabase {
      */
     @Override
     public void open() throws Exception {
+
         writeLock.lock();
         try {
             // Erase any previous groups and users
@@ -425,13 +425,10 @@ public class MemoryUserDatabase implements UserDatabase {
 
             String pathName = getPathname();
             URI uri = ConfigFileLoader.getURI(pathName);
-            URLConnection uConn = null;
+            URL url = uri.toURL();
+            URLConnection uConn = url.openConnection();
 
-            try {
-                URL url = uri.toURL();
-                uConn = url.openConnection();
-
-                InputStream is = uConn.getInputStream();
+            try (InputStream is = uConn.getInputStream()) {
                 this.lastModified = uConn.getLastModified();
 
                 // Construct a digester to read the XML input file
@@ -453,22 +450,14 @@ public class MemoryUserDatabase implements UserDatabase {
                 digester.parse(is);
             } catch (IOException ioe) {
                 log.error(sm.getString("memoryUserDatabase.fileNotFound", pathName));
-            } catch (Exception e) {
-                // Fail safe on error
-                users.clear();
-                groups.clear();
-                roles.clear();
-                throw e;
-            } finally {
-                if (uConn != null) {
-                    try {
-                        // Can't close a uConn directly. Have to do it like this.
-                        uConn.getInputStream().close();
-                    } catch (IOException ioe) {
-                        log.warn(sm.getString("memoryUserDatabase.fileClose", pathname), ioe);
-                    }
-                }
             }
+        } catch (Exception e) {
+            // Fail safe on error
+            users.clear();
+            groups.clear();
+            roles.clear();
+
+            throw e;
         } finally {
             writeLock.unlock();
         }
@@ -583,7 +572,7 @@ public class MemoryUserDatabase implements UserDatabase {
         writeLock.lock();
         try {
             try (FileOutputStream fos = new FileOutputStream(fileNew);
-                    OutputStreamWriter osw = new OutputStreamWriter(fos, StandardCharsets.UTF_8);
+                    OutputStreamWriter osw = new OutputStreamWriter(fos, "UTF8");
                     PrintWriter writer = new PrintWriter(osw)) {
 
                 // Print the file prolog
@@ -665,16 +654,16 @@ public class MemoryUserDatabase implements UserDatabase {
     }
 
 
+    @Override
     public void backgroundProcess() {
         if (!watchSource) {
             return;
         }
 
         URI uri = ConfigFileLoader.getURI(getPathname());
-        URLConnection uConn = null;
         try {
             URL url = uri.toURL();
-            uConn = url.openConnection();
+            URLConnection uConn = url.openConnection();
 
             if (this.lastModified != uConn.getLastModified()) {
                 writeLock.lock();
@@ -694,20 +683,6 @@ public class MemoryUserDatabase implements UserDatabase {
             }
         } catch (Exception ioe) {
             log.error(sm.getString("memoryUserDatabase.reloadError", id, uri), ioe);
-        } finally {
-            if (uConn != null) {
-                try {
-                    // Can't close a uConn directly. Have to do it like this.
-                    uConn.getInputStream().close();
-                } catch (FileNotFoundException fnfe) {
-                    // The file doesn't exist.
-                    // This has been logged above. No need to log again.
-                    // Set the last modified time to avoid repeated log messages
-                    this.lastModified = 0;
-                } catch (IOException ioe) {
-                    log.warn(sm.getString("memoryUserDatabase.fileClose", pathname), ioe);
-                }
-            }
         }
     }
 
