@@ -54,20 +54,6 @@ import org.apache.tomcat.util.buf.UDecoder;
 import org.apache.tomcat.util.buf.UriUtil;
 import org.apache.tomcat.util.http.RequestUtil;
 
-/**
- * Note: Extra caution should be used when adding a Rewrite Rule. When
- * specifying a regex to match for in a Rewrite Rule, certain regex could allow
- * an attacker to DoS your server, as Java's regex parsing is vulnerable to
- * "catastrophic backtracking" (also known as "Regular expression Denial of
- * Service", or ReDoS). There are some open source tools to help detect
- * vulnerable regex, though in general it is a hard problem. A good defence is
- * to use a regex debugger on your desired regex, and read more on the subject
- * of catastrophic backtracking.
- *
- * @see <a href=
- *      "https://www.owasp.org/index.php/Regular_expression_Denial_of_Service_-_ReDoS">OWASP
- *      ReDoS</a>
- */
 public class RewriteValve extends ValveBase {
 
     /**
@@ -188,12 +174,12 @@ public class RewriteValve extends ValveBase {
                 BufferedReader reader = new BufferedReader(isr)) {
             parse(reader);
         } catch (IOException ioe) {
-            containerLog.error(sm.getString("rewriteValve.closeError"), ioe);
+            containerLog.error("Error closing configuration", ioe);
         } finally {
             try {
                 is.close();
             } catch (IOException e) {
-                containerLog.error(sm.getString("rewriteValve.closeError"), e);
+                containerLog.error("Error closing configuration", e);
             }
         }
 
@@ -273,7 +259,7 @@ public class RewriteValve extends ValveBase {
                     }
                 }
             } catch (IOException e) {
-                containerLog.error(sm.getString("rewriteValve.readError"), e);
+                containerLog.error("Error reading configuration", e);
             }
         }
         this.rules = rules.toArray(new RewriteRule[0]);
@@ -638,7 +624,7 @@ public class RewriteValve extends ValveBase {
                 // RewriteCond TestString CondPattern [Flags]
                 RewriteCond condition = new RewriteCond();
                 if (tokenizer.countTokens() < 2) {
-                    throw new IllegalArgumentException(sm.getString("rewriteValve.invalidLine", line));
+                    throw new IllegalArgumentException("Invalid line: " + line);
                 }
                 condition.setTestString(tokenizer.nextToken());
                 condition.setCondPattern(tokenizer.nextToken());
@@ -658,7 +644,7 @@ public class RewriteValve extends ValveBase {
                 // RewriteRule Pattern Substitution [Flags]
                 RewriteRule rule = new RewriteRule();
                 if (tokenizer.countTokens() < 2) {
-                    throw new IllegalArgumentException(sm.getString("rewriteValve.invalidLine", line));
+                    throw new IllegalArgumentException("Invalid line: " + line);
                 }
                 rule.setPatternString(tokenizer.nextToken());
                 rule.setSubstitutionString(tokenizer.nextToken());
@@ -677,7 +663,7 @@ public class RewriteValve extends ValveBase {
             } else if (token.equals("RewriteMap")) {
                 // RewriteMap name rewriteMapClassName whateverOptionalParameterInWhateverFormat
                 if (tokenizer.countTokens() < 2) {
-                    throw new IllegalArgumentException(sm.getString("rewriteValve.invalidLine", line));
+                    throw new IllegalArgumentException("Invalid line: " + line);
                 }
                 String name = tokenizer.nextToken();
                 String rewriteMapClassName = tokenizer.nextToken();
@@ -686,7 +672,7 @@ public class RewriteValve extends ValveBase {
                     map = (RewriteMap) (Class.forName(
                             rewriteMapClassName).getConstructor().newInstance());
                 } catch (Exception e) {
-                    throw new IllegalArgumentException(sm.getString("rewriteValve.invalidMapClassName", line));
+                    throw new IllegalArgumentException("Invalid map className: " + line);
                 }
                 if (tokenizer.hasMoreTokens()) {
                     map.setParameters(tokenizer.nextToken());
@@ -698,7 +684,7 @@ public class RewriteValve extends ValveBase {
             } else if (token.startsWith("#")) {
                 // it's a comment, ignore it
             } else {
-                throw new IllegalArgumentException(sm.getString("rewriteValve.invalidLine", line));
+                throw new IllegalArgumentException("Invalid line: " + line);
             }
         }
         return null;
@@ -717,7 +703,7 @@ public class RewriteValve extends ValveBase {
         } else if (flag.equals("OR") || flag.equals("ornext")) {
             condition.setOrnext(true);
         } else {
-            throw new IllegalArgumentException(sm.getString("rewriteValve.invalidFlags", line, flag));
+            throw new IllegalArgumentException("Invalid flag in: " + line + " flags: " + flag);
         }
     }
 
@@ -742,7 +728,7 @@ public class RewriteValve extends ValveBase {
             }
             StringTokenizer tokenizer = new StringTokenizer(flag, ":");
             if (tokenizer.countTokens() < 2) {
-                throw new IllegalArgumentException(sm.getString("rewriteValve.invalidFlags", line, flag));
+                throw new IllegalArgumentException("Invalid flag in: " + line);
             }
             rule.setCookieName(tokenizer.nextToken());
             rule.setCookieValue(tokenizer.nextToken());
@@ -753,7 +739,7 @@ public class RewriteValve extends ValveBase {
                 try {
                     rule.setCookieLifetime(Integer.parseInt(tokenizer.nextToken()));
                 } catch (NumberFormatException e) {
-                    throw new IllegalArgumentException(sm.getString("rewriteValve.invalidFlags", line, flag), e);
+                    throw new IllegalArgumentException("Invalid flag in: " + line, e);
                 }
             }
             if (tokenizer.hasMoreTokens()) {
@@ -774,7 +760,7 @@ public class RewriteValve extends ValveBase {
             }
             int pos = flag.indexOf(':');
             if (pos == -1 || (pos + 1) == flag.length()) {
-                throw new IllegalArgumentException(sm.getString("rewriteValve.invalidFlags", line, flag));
+                throw new IllegalArgumentException("Invalid flag in: " + line);
             }
             rule.addEnvName(flag.substring(0, pos));
             rule.addEnvValue(flag.substring(pos + 1));
@@ -797,7 +783,7 @@ public class RewriteValve extends ValveBase {
         } else if (flag.startsWith("qsappend") || flag.startsWith("QSA")) {
             rule.setQsappend(true);
         } else if (flag.startsWith("qsdiscard") || flag.startsWith("QSD")) {
-            rule.setQsdiscard(true);
+            rule.setQsappend(true);
         } else if (flag.startsWith("redirect") || flag.startsWith("R")) {
             rule.setRedirect(true);
             int redirectCode = HttpServletResponse.SC_FOUND;
@@ -839,7 +825,7 @@ public class RewriteValve extends ValveBase {
             rule.setType(true);
             rule.setTypeValue(flag);
         } else {
-            throw new IllegalArgumentException(sm.getString("rewriteValve.invalidFlags", line, flag));
+            throw new IllegalArgumentException("Invalid flag in: " + line + " flag: " + flag);
         }
     }
 }
