@@ -20,9 +20,8 @@ import java.io.IOException;
 import java.io.InterruptedIOException;
 import java.nio.ByteBuffer;
 import java.util.Iterator;
-import java.util.concurrent.Executor;
+import java.util.concurrent.RejectedExecutionException;
 import java.util.concurrent.atomic.AtomicBoolean;
-import java.util.concurrent.atomic.AtomicReference;
 
 import javax.servlet.RequestDispatcher;
 
@@ -31,7 +30,6 @@ import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
 import org.apache.tomcat.util.http.parser.Host;
 import org.apache.tomcat.util.log.UserDataHelper;
-import org.apache.tomcat.util.net.AbstractEndpoint;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 import org.apache.tomcat.util.net.DispatchType;
 import org.apache.tomcat.util.net.SSLSupport;
@@ -41,16 +39,16 @@ import org.apache.tomcat.util.res.StringManager;
 
 /**
  * Provides functionality and attributes common to all supported protocols
- * (currently HTTP and AJP) for processing a single request/response.
+ * (currently HTTP and AJP).
  */
 public abstract class AbstractProcessor extends AbstractProcessorLight implements ActionHook {
 
     private static final StringManager sm = StringManager.getManager(AbstractProcessor.class);
 
     // Used to avoid useless B2C conversion on the host name.
-    protected char[] hostNameC = new char[0];
+    private char[] hostNameC = new char[0];
 
-    protected Adapter adapter;
+    protected final Adapter adapter;
     protected final AsyncStateMachine asyncStateMachine;
     private volatile long asyncTimeout = -1;
     /*
@@ -63,7 +61,6 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
      * current async generation. This prevents the response mix-up.
      */
     private volatile long asyncTimeoutGeneration = 0;
-    protected final AbstractEndpoint<?> endpoint;
     protected final Request request;
     protected final Response response;
     protected volatile SocketWrapperBase<?> socketWrapper = null;
@@ -77,14 +74,13 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
 
     protected final UserDataHelper userDataHelper;
 
-    public AbstractProcessor(AbstractEndpoint<?> endpoint) {
-        this(endpoint, new Request(), new Response());
+    public AbstractProcessor(Adapter adapter) {
+        this(adapter, new Request(), new Response());
     }
 
 
-    protected AbstractProcessor(AbstractEndpoint<?> endpoint, Request coyoteRequest,
-            Response coyoteResponse) {
-        this.endpoint = endpoint;
+    protected AbstractProcessor(Adapter adapter, Request coyoteRequest, Response coyoteResponse) {
+        this.adapter = adapter;
         asyncStateMachine = new AsyncStateMachine(this);
         request = coyoteRequest;
         response = coyoteResponse;
@@ -94,7 +90,6 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
         userDataHelper = new UserDataHelper(getLog());
     }
 
-
     /**
      * Update the current error state to the new error state if the new error
      * state is more severe than the current error state.
@@ -102,9 +97,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
      * @param t The error which occurred
      */
     protected void setErrorState(ErrorState errorState, Throwable t) {
-        // Use the return value to avoid processing more than one async error
-        // in a single async cycle.
-        boolean setError = response.setError();
+        response.setError();
         boolean blockIo = this.errorState.isIoAllowed() && !errorState.isIoAllowed();
         this.errorState = this.errorState.getMostSevere(errorState);
         // Don't change the status code for IOException since that is almost
@@ -116,10 +109,17 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
         if (t != null) {
             request.setAttribute(RequestDispatcher.ERROR_EXCEPTION, t);
         }
-        if (blockIo && isAsync() && setError) {
-            if (asyncStateMachine.asyncError()) {
-                processSocketEvent(SocketEvent.ERROR, true);
+        if (blockIo && !ContainerThreadMarker.isContainerThread() && isAsync()) {
+            // The error occurred on a non-container thread during async
+            // processing which means not all of the necessary clean-up will
+            // have been completed. Dispatch to a container thread to do the
+            // clean-up. Need to do it this way to ensure that all the necessary
+            // clean-up is performed.
+            asyncStateMachine.asyncMustError();
+            if (getLog().isDebugEnabled()) {
+                getLog().debug(sm.getString("abstractProcessor.nonContainerThreadError"), t);
             }
+            processSocketEvent(SocketEvent.ERROR, true);
         }
     }
 
@@ -135,16 +135,6 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
-    /**
-     * Set the associated adapter.
-     *
-     * @param adapter the new adapter
-     */
-    public void setAdapter(Adapter adapter) {
-        this.adapter = adapter;
-    }
-
-
     /**
      * Get the associated adapter.
      *
@@ -159,7 +149,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
      * Set the socket wrapper being used.
      * @param socketWrapper The socket wrapper
      */
-    protected void setSocketWrapper(SocketWrapperBase<?> socketWrapper) {
+    protected final void setSocketWrapper(SocketWrapperBase<?> socketWrapper) {
         this.socketWrapper = socketWrapper;
     }
 
@@ -179,10 +169,18 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
 
 
     /**
-     * @return the Executor used by the underlying endpoint.
+     * Provides a mechanism to trigger processing on a container thread.
+     *
+     * @param runnable  The task representing the processing that needs to take
+     *                  place on a container thread
      */
-    protected Executor getExecutor() {
-        return endpoint.getExecutor();
+    protected void execute(Runnable runnable) {
+        SocketWrapperBase<?> socketWrapper = this.socketWrapper;
+        if (socketWrapper == null) {
+            throw new RejectedExecutionException(sm.getString("abstractProcessor.noExecute"));
+        } else {
+            socketWrapper.execute(runnable);
+        }
     }
 
 
@@ -199,7 +197,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
 
 
     @Override
-    public final SocketState dispatch(SocketEvent status) throws IOException {
+    public final SocketState dispatch(SocketEvent status) {
 
         if (status == SocketEvent.OPEN_WRITE && response.getWriteListener() != null) {
             asyncStateMachine.asyncOperation();
@@ -253,25 +251,15 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
 
         rp.setStage(org.apache.coyote.Constants.STAGE_ENDED);
 
-        SocketState state;
-
         if (getErrorState().isError()) {
             request.updateCounters();
-            state = SocketState.CLOSED;
+            return SocketState.CLOSED;
         } else if (isAsync()) {
-            state = SocketState.LONG;
+            return SocketState.LONG;
         } else {
             request.updateCounters();
-            state = dispatchEndRequest();
+            return dispatchEndRequest();
         }
-
-        if (getLog().isDebugEnabled()) {
-            getLog().debug("Socket: [" + socketWrapper +
-                    "], Status in: [" + status +
-                    "], State out: [" + state + "]");
-        }
-
-        return state;
     }
 
 
@@ -380,7 +368,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
                     // Validate and write response headers
                     prepareResponse();
                 } catch (IOException e) {
-                    handleIOException(e);
+                    setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                 }
             }
             break;
@@ -389,8 +377,10 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
             action(ActionCode.COMMIT, null);
             try {
                 finishResponse();
+            } catch (CloseNowException cne) {
+                setErrorState(ErrorState.CLOSE_NOW, cne);
             } catch (IOException e) {
-                handleIOException(e);
+                setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
             }
             break;
         }
@@ -403,7 +393,7 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
             try {
                 flush();
             } catch (IOException e) {
-                handleIOException(e);
+                setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                 response.setErrorException(e);
             }
             break;
@@ -575,13 +565,14 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
             break;
         }
         case NB_READ_INTEREST: {
-            AtomicBoolean isReady = (AtomicBoolean)param;
-            isReady.set(isReadyForRead());
+            if (!isRequestBodyFullyRead()) {
+                registerReadInterest();
+            }
             break;
         }
         case NB_WRITE_INTEREST: {
             AtomicBoolean isReady = (AtomicBoolean)param;
-            isReady.set(isReadyForWrite());
+            isReady.set(isReady());
             break;
         }
         case DISPATCH_READ: {
@@ -614,34 +605,21 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
             break;
         }
 
-        // Identifiers associated with multiplexing protocols like HTTP/2
-        case CONNECTION_ID: {
-            @SuppressWarnings("unchecked")
-            AtomicReference<Object> result = (AtomicReference<Object>) param;
-            result.set(getConnectionID());
+        // Servlet 4.0 Trailers
+        case IS_TRAILER_FIELDS_READY: {
+            AtomicBoolean result = (AtomicBoolean) param;
+            result.set(isTrailerFieldsReady());
             break;
         }
-        case STREAM_ID: {
-            @SuppressWarnings("unchecked")
-            AtomicReference<Object> result = (AtomicReference<Object>) param;
-            result.set(getStreamID());
+        case IS_TRAILER_FIELDS_SUPPORTED: {
+            AtomicBoolean result = (AtomicBoolean) param;
+            result.set(isTrailerFieldsSupported());
             break;
         }
         }
     }
 
 
-    private void handleIOException (IOException ioe) {
-        if (ioe instanceof CloseNowException) {
-            // Close the channel but keep the connection open
-            setErrorState(ErrorState.CLOSE_NOW, ioe);
-        } else {
-            // Close the connection and all channels within that connection
-            setErrorState(ErrorState.CLOSE_CONNECTION_NOW, ioe);
-        }
-    }
-
-
     /**
      * Perform any necessary processing for a non-blocking read before
      * dispatching to the adapter.
@@ -651,13 +629,6 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
-    /**
-     * {@inheritDoc}
-     * <p>
-     * Sub-classes of this base class represent a single request/response pair.
-     * The timeout to be processed is, therefore, the Servlet asynchronous
-     * processing timeout.
-     */
     @Override
     public void timeoutAsync(long now) {
         if (now < 0) {
@@ -669,10 +640,6 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
                 if ((now - asyncStart) > asyncTimeout) {
                     doTimeoutAsync();
                 }
-            } else if (!asyncStateMachine.isAvailable()) {
-                // Timeout the async process if the associated web application
-                // is no longer running.
-                doTimeoutAsync();
             }
         }
     }
@@ -814,26 +781,13 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
-    protected boolean isReadyForRead() {
-        if (available(true) > 0) {
-            return true;
-        }
-
-        if (!isRequestBodyFullyRead()) {
-            registerReadInterest();
-        }
-
-        return false;
-    }
-
-
     protected abstract boolean isRequestBodyFullyRead();
 
 
     protected abstract void registerReadInterest();
 
 
-    protected abstract boolean isReadyForWrite();
+    protected abstract boolean isReady();
 
 
     protected void executeDispatches() {
@@ -947,27 +901,18 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
     }
 
 
-    /**
-     * Protocols that support multiplexing (e.g. HTTP/2) should override this
-     * method and return the appropriate ID.
-     *
-     * @return The stream ID associated with this request or {@code null} if a
-     *         multiplexing protocol is not being used
-      */
-    protected Object getConnectionID() {
-        return null;
-    }
+    protected abstract boolean isTrailerFieldsReady();
 
 
     /**
-     * Protocols that support multiplexing (e.g. HTTP/2) should override this
-     * method and return the appropriate ID.
+     * Protocols that support trailer fields should override this method and
+     * return {@code true}.
      *
-     * @return The stream ID associated with this request or {@code null} if a
-     *         multiplexing protocol is not being used
+     * @return {@code true} if trailer fields are supported by this processor,
+     *         otherwise {@code false}.
      */
-    protected Object getStreamID() {
-        return null;
+    protected boolean isTrailerFieldsSupported() {
+        return false;
     }
 
 
@@ -983,30 +928,12 @@ public abstract class AbstractProcessor extends AbstractProcessorLight implement
      */
     protected abstract boolean flushBufferedWrite() throws IOException ;
 
-
     /**
      * Perform any necessary clean-up processing if the dispatch resulted in the
      * completion of processing for the current request.
      *
      * @return The state to return for the socket once the clean-up for the
      *         current request has completed
-     *
-     * @throws IOException If an I/O error occurs while attempting to end the
-     *         request
      */
-    protected abstract SocketState dispatchEndRequest() throws IOException;
-
-
-    @Override
-    protected final void logAccess(SocketWrapperBase<?> socketWrapper) throws IOException {
-        // Set the socket wrapper so the access log can read the socket related
-        // information (e.g. client IP)
-        setSocketWrapper(socketWrapper);
-        // Setup the minimal request information
-        request.setStartTime(System.currentTimeMillis());
-        // Setup the minimal response information
-        response.setStatus(400);
-        response.setError();
-        getAdapter().log(request, response, 0);
-    }
+    protected abstract SocketState dispatchEndRequest();
 }
