@@ -25,16 +25,12 @@ import java.nio.ByteBuffer;
 import java.security.NoSuchProviderException;
 import java.security.cert.CertificateFactory;
 import java.security.cert.X509Certificate;
-import java.util.Collections;
-import java.util.HashSet;
-import java.util.Set;
-import java.util.regex.Matcher;
-import java.util.regex.Pattern;
 
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.coyote.AbstractProcessor;
 import org.apache.coyote.ActionCode;
+import org.apache.coyote.Adapter;
 import org.apache.coyote.ErrorState;
 import org.apache.coyote.InputBuffer;
 import org.apache.coyote.OutputBuffer;
@@ -44,9 +40,7 @@ import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
-import org.apache.tomcat.util.http.HttpMessages;
 import org.apache.tomcat.util.http.MimeHeaders;
-import org.apache.tomcat.util.net.AbstractEndpoint;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 import org.apache.tomcat.util.net.ApplicationBufferHandler;
 import org.apache.tomcat.util.net.SSLSupport;
@@ -84,10 +78,6 @@ public class AjpProcessor extends AbstractProcessor {
     private static final byte[] pongMessageArray;
 
 
-    private static final Set<String> javaxAttributes;
-    private static final Set<String> iisTlsAttributes;
-
-
     static {
         // Allocate the end message array
         AjpMessage endMessage = new AjpMessage(16);
@@ -128,31 +118,13 @@ public class AjpProcessor extends AbstractProcessor {
         pongMessageArray = new byte[pongMessage.getLen()];
         System.arraycopy(pongMessage.getBuffer(), 0, pongMessageArray,
                 0, pongMessage.getLen());
-
-        // Build the Set of javax attributes
-        Set<String> s = new HashSet<>();
-        s.add("javax.servlet.request.cipher_suite");
-        s.add("javax.servlet.request.key_size");
-        s.add("javax.servlet.request.ssl_session");
-        s.add("javax.servlet.request.X509Certificate");
-        javaxAttributes= Collections.unmodifiableSet(s);
-
-        Set<String> iis = new HashSet<>();
-        iis.add("CERT_ISSUER");
-        iis.add("CERT_SUBJECT");
-        iis.add("CERT_COOKIE");
-        iis.add("HTTPS_SERVER_SUBJECT");
-        iis.add("CERT_FLAGS");
-        iis.add("HTTPS_SECRETKEYSIZE");
-        iis.add("CERT_SERIALNUMBER");
-        iis.add("HTTPS_SERVER_ISSUER");
-        iis.add("HTTPS_KEYSIZE");
-        iisTlsAttributes = Collections.unmodifiableSet(iis);
     }
 
 
     // ----------------------------------------------------- Instance Variables
 
+    private final AbstractAjpProtocol<?> protocol;
+
 
     /**
      * GetBody message array. Not static like the other message arrays since the
@@ -266,14 +238,14 @@ public class AjpProcessor extends AbstractProcessor {
 
     // ------------------------------------------------------------ Constructor
 
-    public AjpProcessor(int packetSize, AbstractEndpoint<?> endpoint) {
-
-        super(endpoint);
+    public AjpProcessor(AbstractAjpProtocol<?> protocol, Adapter adapter) {
+        super(adapter);
+        this.protocol = protocol;
 
+        int packetSize = protocol.getPacketSize();
         // Calculate maximum chunk size as packetSize may have been changed from
         // the default (Constants.MAX_PACKET_SIZE)
-        this.outputMaxChunkSize =
-                Constants.MAX_SEND_SIZE + packetSize - Constants.MAX_PACKET_SIZE;
+        this.outputMaxChunkSize = packetSize - Constants.SEND_HEAD_LEN;
 
         request.setInputBuffer(new SocketInputBuffer());
 
@@ -297,96 +269,6 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
-    // ------------------------------------------------------------- Properties
-
-
-    /**
-     * Send AJP flush packet when flushing.
-     * An flush packet is a zero byte AJP13 SEND_BODY_CHUNK
-     * packet. mod_jk and mod_proxy_ajp interprete this as
-     * a request to flush data to the client.
-     * AJP always does flush at the and of the response, so if
-     * it is not important, that the packets get streamed up to
-     * the client, do not use extra flush packets.
-     * For compatibility and to stay on the safe side, flush
-     * packets are enabled by default.
-     */
-    protected boolean ajpFlush = true;
-    public boolean getAjpFlush() { return ajpFlush; }
-    public void setAjpFlush(boolean ajpFlush) {
-        this.ajpFlush = ajpFlush;
-    }
-
-
-    /**
-     * The number of milliseconds Tomcat will wait for a subsequent request
-     * before closing the connection. The default is -1 which is an infinite
-     * timeout.
-     */
-    private int keepAliveTimeout = -1;
-    public int getKeepAliveTimeout() { return keepAliveTimeout; }
-    public void setKeepAliveTimeout(int timeout) { keepAliveTimeout = timeout; }
-
-
-    /**
-     * Use Tomcat authentication ?
-     */
-    private boolean tomcatAuthentication = true;
-    public boolean getTomcatAuthentication() { return tomcatAuthentication; }
-    public void setTomcatAuthentication(boolean tomcatAuthentication) {
-        this.tomcatAuthentication = tomcatAuthentication;
-    }
-
-
-    /**
-     * Use Tomcat authorization ?
-     */
-    private boolean tomcatAuthorization = false;
-    public boolean getTomcatAuthorization() { return tomcatAuthorization; }
-    public void setTomcatAuthorization(boolean tomcatAuthorization) {
-        this.tomcatAuthorization = tomcatAuthorization;
-    }
-
-
-    /**
-     * Required secret.
-     */
-    private String secret = null;
-    @Deprecated
-    public void setRequiredSecret(String requiredSecret) {
-        setSecret(requiredSecret);
-    }
-    public void setSecret(String secret) {
-        this.secret = secret;
-    }
-
-
-    /**
-     * When client certificate information is presented in a form other than
-     * instances of {@link java.security.cert.X509Certificate} it needs to be
-     * converted before it can be used and this property controls which JSSE
-     * provider is used to perform the conversion.
-     */
-    private String clientCertProvider = null;
-    public String getClientCertProvider() { return clientCertProvider; }
-    public void setClientCertProvider(String clientCertProvider) {
-        this.clientCertProvider = clientCertProvider;
-    }
-
-
-    @Deprecated
-    private boolean sendReasonPhrase = false;
-    @Deprecated
-    void setSendReasonPhrase(boolean sendReasonPhrase) {
-        this.sendReasonPhrase = sendReasonPhrase;
-    }
-
-
-    private Pattern allowedRequestAttributesPattern;
-    public void setAllowedRequestAttributesPattern(Pattern allowedRequestAttributesPattern) {
-        this.allowedRequestAttributesPattern = allowedRequestAttributesPattern;
-    }
-
     // --------------------------------------------------------- Public Methods
 
     @Override
@@ -414,16 +296,10 @@ public class AjpProcessor extends AbstractProcessor {
 
     @Override
     protected SocketState dispatchEndRequest() {
-        // Set keep alive timeout for next request if enabled
-        if (keepAliveTimeout > 0) {
-            socketWrapper.setReadTimeout(keepAliveTimeout);
-        }
+        // Set keep alive timeout for next request
+        socketWrapper.setReadTimeout(protocol.getKeepAliveTimeout());
         recycle();
-        if (endpoint.isPaused()) {
-            return SocketState.CLOSED;
-        } else {
-            return SocketState.OPEN;
-        }
+        return SocketState.OPEN;
     }
 
 
@@ -436,31 +312,26 @@ public class AjpProcessor extends AbstractProcessor {
         // Setting up the socket
         this.socketWrapper = socket;
 
-        int soTimeout = endpoint.getConnectionTimeout();
         boolean cping = false;
-        // Expected to block on the first read as there should be at least one
-        // AJP message to read.
-        boolean firstRead = true;
+        boolean keptAlive = false;
 
-        while (!getErrorState().isError() && !endpoint.isPaused()) {
+        while (!getErrorState().isError() && !protocol.isPaused()) {
             // Parsing the request header
             try {
                 // Get first message of the request
-                if (!readMessage(requestHeaderMessage, firstRead)) {
+                if (!readMessage(requestHeaderMessage, !keptAlive)) {
                     break;
                 }
-                firstRead = false;
 
-                // Set back timeout if keep alive timeout is enabled
-                if (keepAliveTimeout > 0) {
-                    socketWrapper.setReadTimeout(soTimeout);
-                }
+                // Processing the request so make sure the connection rather
+                // than keep-alive timeout is used
+                socketWrapper.setReadTimeout(protocol.getConnectionTimeout());
 
                 // Check message type, process right away and break if
                 // not regular request processing
                 int type = requestHeaderMessage.getByte();
                 if (type == Constants.JK_AJP13_CPING_REQUEST) {
-                    if (endpoint.isPaused()) {
+                    if (protocol.isPaused()) {
                         recycle();
                         break;
                     }
@@ -469,9 +340,6 @@ public class AjpProcessor extends AbstractProcessor {
                         socketWrapper.write(true, pongMessageArray, 0, pongMessageArray.length);
                         socketWrapper.flush(true);
                     } catch (IOException e) {
-                        if (getLog().isDebugEnabled()) {
-                            getLog().debug("Pong message failed", e);
-                        }
                         setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
                     }
                     recycle();
@@ -485,6 +353,7 @@ public class AjpProcessor extends AbstractProcessor {
                     setErrorState(ErrorState.CLOSE_CONNECTION_NOW, null);
                     break;
                 }
+                keptAlive = true;
                 request.setStartTime(System.currentTimeMillis());
             } catch (IOException e) {
                 setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
@@ -511,7 +380,7 @@ public class AjpProcessor extends AbstractProcessor {
                 }
             }
 
-            if (getErrorState().isIoAllowed() && !cping && endpoint.isPaused()) {
+            if (getErrorState().isIoAllowed() && !cping && protocol.isPaused()) {
                 // 503 - Service unavailable
                 response.setStatus(503);
                 setErrorState(ErrorState.CLOSE_CLEAN, null);
@@ -560,17 +429,16 @@ public class AjpProcessor extends AbstractProcessor {
             request.updateCounters();
 
             rp.setStage(org.apache.coyote.Constants.STAGE_KEEPALIVE);
-            // Set keep alive timeout for next request if enabled
-            if (keepAliveTimeout > 0) {
-                socketWrapper.setReadTimeout(keepAliveTimeout);
-            }
+
+            // Set keep alive timeout for next request
+            socketWrapper.setReadTimeout(protocol.getKeepAliveTimeout());
 
             recycle();
         }
 
         rp.setStage(org.apache.coyote.Constants.STAGE_ENDED);
 
-        if (getErrorState().isError() || endpoint.isPaused()) {
+        if (getErrorState().isError() || protocol.isPaused()) {
             return SocketState.CLOSED;
         } else {
             if (isAsync()) {
@@ -768,7 +636,7 @@ public class AjpProcessor extends AbstractProcessor {
         MimeHeaders headers = request.getMimeHeaders();
 
         // Set this every time in case limit has been changed via JMX
-        headers.setLimit(endpoint.getMaxHeaderCount());
+        headers.setLimit(protocol.getMaxHeaderCount());
 
         boolean contentLengthSet = false;
         int hCount = requestHeaderMessage.getInt();
@@ -825,7 +693,8 @@ public class AjpProcessor extends AbstractProcessor {
         }
 
         // Decode extra attributes
-        boolean secretPresentInRequest = false;
+        String requiredSecret = protocol.getRequiredSecret();
+        boolean secret = false;
         byte attributeCode;
         while ((attributeCode = requestHeaderMessage.getByte())
                 != Constants.SC_A_ARE_DONE) {
@@ -854,28 +723,8 @@ public class AjpProcessor extends AbstractProcessor {
                     }
                 } else if(n.equals(Constants.SC_A_SSL_PROTOCOL)) {
                     request.setAttribute(SSLSupport.PROTOCOL_VERSION_KEY, v);
-                } else if (n.equals("JK_LB_ACTIVATION")) {
-                    request.setAttribute(n, v);
-                } else if (javaxAttributes.contains(n)) {
-                    request.setAttribute(n, v);
-                } else if (iisTlsAttributes.contains(n)) {
-                    // Allow IIS TLS attributes
-                    request.setAttribute(n, v);
                 } else {
-                    // All 'known' attributes will be processed by the previous
-                    // blocks. Any remaining attribute is an 'arbitrary' one.
-                    if (allowedRequestAttributesPattern == null) {
-                        response.setStatus(403);
-                        setErrorState(ErrorState.CLOSE_CLEAN, null);
-                    } else {
-                        Matcher m = allowedRequestAttributesPattern.matcher(n);
-                        if (m.matches()) {
-                            request.setAttribute(n, v);
-                        } else {
-                            response.setStatus(403);
-                            setErrorState(ErrorState.CLOSE_CLEAN, null);
-                        }
-                    }
+                    request.setAttribute(n, v );
                 }
                 break;
 
@@ -890,7 +739,8 @@ public class AjpProcessor extends AbstractProcessor {
                 break;
 
             case Constants.SC_A_REMOTE_USER :
-                if (tomcatAuthorization || !tomcatAuthentication) {
+                boolean tomcatAuthorization  = protocol.getTomcatAuthorization();
+                if (tomcatAuthorization || !protocol.getTomcatAuthentication()) {
                     // Implies tomcatAuthentication == false
                     requestHeaderMessage.getBytes(request.getRemoteUser());
                     request.setRemoteUserNeedsAuthorization(tomcatAuthorization);
@@ -901,7 +751,7 @@ public class AjpProcessor extends AbstractProcessor {
                 break;
 
             case Constants.SC_A_AUTH_TYPE :
-                if (tomcatAuthentication) {
+                if (protocol.getTomcatAuthentication()) {
                     // ignore server
                     requestHeaderMessage.getBytes(tmpMB);
                 } else {
@@ -946,9 +796,9 @@ public class AjpProcessor extends AbstractProcessor {
 
             case Constants.SC_A_SECRET:
                 requestHeaderMessage.getBytes(tmpMB);
-                if (secret != null && secret.length() > 0) {
-                    secretPresentInRequest = true;
-                    if (!tmpMB.equals(secret)) {
+                if (requiredSecret != null) {
+                    secret = true;
+                    if (!tmpMB.equals(requiredSecret)) {
                         response.setStatus(403);
                         setErrorState(ErrorState.CLOSE_CLEAN, null);
                     }
@@ -964,7 +814,7 @@ public class AjpProcessor extends AbstractProcessor {
         }
 
         // Check if secret was submitted if required
-        if (secret != null && secret.length() > 0 && !secretPresentInRequest) {
+        if ((requiredSecret != null) && !secret) {
             response.setStatus(403);
             setErrorState(ErrorState.CLOSE_CLEAN, null);
         }
@@ -1039,7 +889,6 @@ public class AjpProcessor extends AbstractProcessor {
      * When committing the response, we have to validate the set of headers, as
      * well as setup the response filters.
      */
-    @SuppressWarnings("deprecation")
     @Override
     protected final void prepareResponse() throws IOException {
 
@@ -1069,26 +918,9 @@ public class AjpProcessor extends AbstractProcessor {
 
         // HTTP header contents
         responseMessage.appendInt(statusCode);
-        if (sendReasonPhrase) {
-            String message = null;
-            if (org.apache.coyote.Constants.USE_CUSTOM_STATUS_MSG_IN_HEADER &&
-                    HttpMessages.isSafeInHttpHeader(response.getMessage())) {
-                message = response.getMessage();
-            }
-            if (message == null) {
-                message = HttpMessages.getInstance(
-                        response.getLocale()).getMessage(response.getStatus());
-            }
-            if (message == null) {
-                // mod_jk + httpd 2.x fails with a null status message - bug 45026
-                message = Integer.toString(response.getStatus());
-            }
-            tmpMB.setString(message);
-        } else {
-            // Reason phrase is optional but mod_jk + httpd 2.x fails with a null
-            // reason phrase - bug 45026
-            tmpMB.setString(Integer.toString(response.getStatus()));
-        }
+        // Reason phrase is optional but mod_jk + httpd 2.x fails with a null
+        // reason phrase - bug 45026
+        tmpMB.setString(Integer.toString(response.getStatus()));
         responseMessage.appendBytes(tmpMB);
 
         // Special headers
@@ -1138,7 +970,7 @@ public class AjpProcessor extends AbstractProcessor {
         // non-blocking writes.
         // TODO Validate the assertion above
         if (!responseFinished) {
-            if (ajpFlush) {
+            if (protocol.getAjpFlush()) {
                 // Send the flush message
                 socketWrapper.write(true, flushMessageArray, 0, flushMessageArray.length);
             }
@@ -1263,7 +1095,7 @@ public class AjpProcessor extends AbstractProcessor {
             // Fill the  elements.
             try {
                 CertificateFactory cf;
-                String clientCertProvider = getClientCertProvider();
+                String clientCertProvider = protocol.getClientCertProvider();
                 if (clientCertProvider == null) {
                     cf = CertificateFactory.getInstance("X.509");
                 } else {
@@ -1308,11 +1140,19 @@ public class AjpProcessor extends AbstractProcessor {
 
 
     @Override
-    protected final boolean isReadyForWrite() {
+    protected final boolean isReady() {
         return responseMsgPos == -1 && socketWrapper.isReadyForWrite();
     }
 
 
+    @Override
+    protected boolean isTrailerFieldsReady() {
+        // AJP does not support trailers so return true so app can request the
+        // trailers and find out that there are none.
+        return true;
+    }
+
+
     /**
      * Read at least the specified amount of bytes, and place them
      * in the input buffer. Note that if any data is available to read then this
@@ -1349,36 +1189,6 @@ public class AjpProcessor extends AbstractProcessor {
     }
 
 
-    /**
-     * @deprecated Unused. Will be removed in Tomcat 9. Use
-     *             {@link #writeData(ByteBuffer)}
-     */
-    @Deprecated
-    private void writeData(ByteChunk chunk) throws IOException {
-        boolean blocking = (response.getWriteListener() == null);
-
-        int len = chunk.getLength();
-        int off = 0;
-
-        // Write this chunk
-        while (len > 0) {
-            int thisTime = Math.min(len, outputMaxChunkSize);
-
-            responseMessage.reset();
-            responseMessage.appendByte(Constants.JK_AJP13_SEND_BODY_CHUNK);
-            responseMessage.appendBytes(chunk.getBytes(), chunk.getOffset() + off, thisTime);
-            responseMessage.end();
-            socketWrapper.write(blocking, responseMessage.getBuffer(), 0, responseMessage.getLen());
-            socketWrapper.flush(blocking);
-
-            len -= thisTime;
-            off += thisTime;
-        }
-
-        bytesWritten += off;
-    }
-
-
     private void writeData(ByteBuffer chunk) throws IOException {
         boolean blocking = (response.getWriteListener() == null);
 
@@ -1424,28 +1234,6 @@ public class AjpProcessor extends AbstractProcessor {
      */
     protected class SocketInputBuffer implements InputBuffer {
 
-        /**
-         * @deprecated Unused. Will be removed in Tomcat 9. Use
-         *             {@link #doRead(ApplicationBufferHandler)}
-         */
-        @Deprecated
-        @Override
-        public int doRead(ByteChunk chunk) throws IOException {
-
-            if (endOfStream) {
-                return -1;
-            }
-            if (empty) {
-                if (!refillReadBuffer(true)) {
-                    return -1;
-                }
-            }
-            ByteChunk bc = bodyBytes.getByteChunk();
-            chunk.setBytes(bc.getBuffer(), bc.getStart(), bc.getLength());
-            empty = true;
-            return chunk.getLength();
-        }
-
         @Override
         public int doRead(ApplicationBufferHandler handler) throws IOException {
 
@@ -1473,29 +1261,6 @@ public class AjpProcessor extends AbstractProcessor {
      */
     protected class SocketOutputBuffer implements OutputBuffer {
 
-        /**
-         * @deprecated Unused. Will be removed in Tomcat 9. Use
-         *             {@link #doWrite(ByteBuffer)}
-         */
-        @Deprecated
-        @Override
-        public int doWrite(ByteChunk chunk) throws IOException {
-
-            if (!response.isCommitted()) {
-                // Validate and write response headers
-                try {
-                    prepareResponse();
-                } catch (IOException e) {
-                    setErrorState(ErrorState.CLOSE_CONNECTION_NOW, e);
-                }
-            }
-
-            if (!swallowResponse) {
-                writeData(chunk);
-            }
-            return chunk.getLength();
-        }
-
         @Override
         public int doWrite(ByteBuffer chunk) throws IOException {
 
