@@ -19,16 +19,15 @@ package org.apache.coyote.http11;
 import java.io.IOException;
 import java.io.InterruptedIOException;
 import java.nio.ByteBuffer;
-import java.util.ArrayList;
-import java.util.HashSet;
-import java.util.List;
-import java.util.Set;
+import java.util.Enumeration;
+import java.util.Locale;
 import java.util.regex.Pattern;
 
 import javax.servlet.http.HttpServletResponse;
 
 import org.apache.coyote.AbstractProcessor;
 import org.apache.coyote.ActionCode;
+import org.apache.coyote.Adapter;
 import org.apache.coyote.ErrorState;
 import org.apache.coyote.Request;
 import org.apache.coyote.RequestInfo;
@@ -47,15 +46,13 @@ import org.apache.coyote.http11.upgrade.InternalHttpUpgradeHandler;
 import org.apache.juli.logging.Log;
 import org.apache.juli.logging.LogFactory;
 import org.apache.tomcat.util.ExceptionUtils;
+import org.apache.tomcat.util.buf.Ascii;
 import org.apache.tomcat.util.buf.ByteChunk;
 import org.apache.tomcat.util.buf.MessageBytes;
-import org.apache.tomcat.util.buf.StringUtils;
 import org.apache.tomcat.util.http.FastHttpDateFormat;
 import org.apache.tomcat.util.http.MimeHeaders;
 import org.apache.tomcat.util.http.parser.HttpParser;
-import org.apache.tomcat.util.http.parser.TokenList;
 import org.apache.tomcat.util.log.UserDataHelper;
-import org.apache.tomcat.util.net.AbstractEndpoint;
 import org.apache.tomcat.util.net.AbstractEndpoint.Handler.SocketState;
 import org.apache.tomcat.util.net.SSLSupport;
 import org.apache.tomcat.util.net.SendfileDataBase;
@@ -76,17 +73,16 @@ public class Http11Processor extends AbstractProcessor {
 
     private final AbstractHttp11Protocol<?> protocol;
 
-
     /**
      * Input.
      */
-    protected final Http11InputBuffer inputBuffer;
+    private final Http11InputBuffer inputBuffer;
 
 
     /**
      * Output.
      */
-    protected final Http11OutputBuffer outputBuffer;
+    private final Http11OutputBuffer outputBuffer;
 
 
     private final HttpParser httpParser;
@@ -102,97 +98,65 @@ public class Http11Processor extends AbstractProcessor {
     /**
      * Keep-alive.
      */
-    protected volatile boolean keepAlive = true;
+    private volatile boolean keepAlive = true;
 
 
     /**
      * Flag used to indicate that the socket should be kept open (e.g. for keep
      * alive or send file.
      */
-    protected boolean openSocket = false;
+    private boolean openSocket = false;
 
 
     /**
      * Flag that indicates if the request headers have been completely read.
      */
-    protected boolean readComplete = true;
+    private boolean readComplete = true;
 
     /**
      * HTTP/1.1 flag.
      */
-    protected boolean http11 = true;
+    private boolean http11 = true;
 
 
     /**
      * HTTP/0.9 flag.
      */
-    protected boolean http09 = false;
+    private boolean http09 = false;
 
 
     /**
      * Content delimiter for the request (if false, the connection will
      * be closed at the end of the request).
      */
-    protected boolean contentDelimitation = true;
-
-
-    /**
-     * Regular expression that defines the restricted user agents.
-     */
-    protected Pattern restrictedUserAgents = null;
-
-
-    /**
-     * Maximum number of Keep-Alive requests to honor.
-     */
-    protected int maxKeepAliveRequests = -1;
-
-
-    /**
-     * Maximum timeout on uploads. 5 minutes as in Apache HTTPD server.
-     */
-    protected int connectionUploadTimeout = 300000;
-
-
-    /**
-     * Flag to disable setting a different time-out on uploads.
-     */
-    protected boolean disableUploadTimeout = false;
-
-
-    /**
-     * Max saved post size.
-     */
-    protected int maxSavePostSize = 4 * 1024;
+    private boolean contentDelimitation = true;
 
 
     /**
      * Instance of the new protocol to use after the HTTP connection has been
      * upgraded.
      */
-    protected UpgradeToken upgradeToken = null;
+    private UpgradeToken upgradeToken = null;
 
 
     /**
      * Sendfile data.
      */
-    protected SendfileDataBase sendfileData = null;
+    private SendfileDataBase sendfileData = null;
 
 
-    @SuppressWarnings("deprecation")
-    public Http11Processor(AbstractHttp11Protocol<?> protocol, AbstractEndpoint<?> endpoint) {
-        super(endpoint);
+    public Http11Processor(AbstractHttp11Protocol<?> protocol, Adapter adapter) {
+        super(adapter);
         this.protocol = protocol;
 
         httpParser = new HttpParser(protocol.getRelaxedPathChars(),
                 protocol.getRelaxedQueryChars());
 
         inputBuffer = new Http11InputBuffer(request, protocol.getMaxHttpHeaderSize(),
-                protocol.getRejectIllegalHeader(), httpParser);
+                protocol.getRejectIllegalHeaderName(), httpParser);
         request.setInputBuffer(inputBuffer);
 
-        outputBuffer = new Http11OutputBuffer(response, protocol.getMaxHttpHeaderSize(),
-                protocol.getSendReasonPhrase());
+        outputBuffer = new Http11OutputBuffer(response, protocol.getMaxHttpHeaderSize());
         response.setOutputBuffer(outputBuffer);
 
         // Create and add the identity filters.
@@ -212,7 +176,7 @@ public class Http11Processor extends AbstractProcessor {
         // Create and add buffered input filter
         inputBuffer.addFilter(new BufferedInputFilter());
 
-        // Create and add the gzip filters.
+        // Create and add the chunked filters.
         //inputBuffer.addFilter(new GzipInputFilter());
         outputBuffer.addFilter(new GzipOutputFilter());
 
@@ -221,211 +185,35 @@ public class Http11Processor extends AbstractProcessor {
 
 
     /**
-     * Set compression level.
-     *
-     * @param compression One of <code>on</code>, <code>force</code>,
-     *                    <code>off</code> or the minimum compression size in
-     *                    bytes which implies <code>on</code>
-     *
-     * @deprecated Use {@link Http11Protocol#setCompression(String)}
+     * Specialized utility method: find a sequence of lower case bytes inside
+     * a ByteChunk.
      */
-    @Deprecated
-    public void setCompression(String compression) {
-        protocol.setCompression(compression);
-    }
+    private static int findBytes(ByteChunk bc, byte[] b) {
 
-    /**
-     * Set Minimum size to trigger compression.
-     *
-     * @param compressionMinSize The minimum content length required for
-     *                           compression in bytes
-     *
-     * @deprecated Use {@link Http11Protocol#setCompressionMinSize(int)}
-     */
-    @Deprecated
-    public void setCompressionMinSize(int compressionMinSize) {
-        protocol.setCompressionMinSize(compressionMinSize);
-    }
+        byte first = b[0];
+        byte[] buff = bc.getBuffer();
+        int start = bc.getStart();
+        int end = bc.getEnd();
 
+        // Look for first char
+        int srcEnd = b.length;
 
-    /**
-     * Set no compression user agent pattern. Regular expression as supported
-     * by {@link Pattern}. e.g.: <code>gorilla|desesplorer|tigrus</code>.
-     *
-     * @param noCompressionUserAgents The regular expression for user agent
-     *                                strings for which compression should not
-     *                                be applied
-     *
-     * @deprecated Use {@link Http11Protocol#setNoCompressionUserAgents(String)}
-     */
-    @Deprecated
-    public void setNoCompressionUserAgents(String noCompressionUserAgents) {
-        protocol.setNoCompressionUserAgents(noCompressionUserAgents);
-    }
-
-
-    /**
-     * @param compressibleMimeTypes See
-     *        {@link Http11Processor#setCompressibleMimeTypes(String[])}
-     *
-     * @deprecated Use {@link Http11Protocol#setCompressibleMimeType(String)}
-     */
-    @Deprecated
-    public void setCompressableMimeTypes(String[] compressibleMimeTypes) {
-        setCompressibleMimeTypes(compressibleMimeTypes);
-    }
-
-
-    /**
-     * Set compressible mime-type list.
-     *
-     * @param compressibleMimeTypes MIME types for which compression should be
-     *                              enabled
-     *
-     * @deprecated Use {@link Http11Protocol#setCompressibleMimeType(String)}
-     */
-    @Deprecated
-    public void setCompressibleMimeTypes(String[] compressibleMimeTypes) {
-        // Conversion from array to comma separated string and back to array in
-        // CompressionConfig isn't ideal but it is better than adding a direct
-        // setter only to immediately deprecate it as it doesn't exist in 9.0.x
-        // given that Tomcat doesn't call this method in normal usage.
-        protocol.setCompressibleMimeType(StringUtils.join(compressibleMimeTypes));
-    }
-
-
-    /**
-     * Return compression level.
-     *
-     * @return The current compression level in string form (off/on/force)
-     *
-     * @deprecated Use {@link Http11Protocol#getCompression()}
-     */
-    @Deprecated
-    public String getCompression() {
-        return protocol.getCompression();
-    }
-
-
-    /**
-     * Set restricted user agent list (which will downgrade the connector
-     * to HTTP/1.0 mode). Regular expression as supported by {@link Pattern}.
-     *
-     * @param restrictedUserAgents The regular expression as supported by
-     *                             {@link Pattern} for the user agents e.g.
-     *                             "gorilla|desesplorer|tigrus"
-     */
-    public void setRestrictedUserAgents(String restrictedUserAgents) {
-        if (restrictedUserAgents == null ||
-                restrictedUserAgents.length() == 0) {
-            this.restrictedUserAgents = null;
-        } else {
-            this.restrictedUserAgents = Pattern.compile(restrictedUserAgents);
+        for (int i = start; i <= (end - srcEnd); i++) {
+            if (Ascii.toLower(buff[i]) != first) {
+                continue;
+            }
+            // found first char, now look for a match
+            int myPos = i+1;
+            for (int srcPos = 1; srcPos < srcEnd;) {
+                if (Ascii.toLower(buff[myPos++]) != b[srcPos++]) {
+                    break;
+                }
+                if (srcPos == srcEnd) {
+                    return i - start; // found it
+                }
+            }
         }
-    }
-
-
-    /**
-     * Set the maximum number of Keep-Alive requests to allow.
-     * This is to safeguard from DoS attacks. Setting to a negative
-     * value disables the limit.
-     *
-     * @param mkar The new maximum number of Keep-Alive requests allowed
-     */
-    public void setMaxKeepAliveRequests(int mkar) {
-        maxKeepAliveRequests = mkar;
-    }
-
-
-    /**
-     * Get the maximum number of Keep-Alive requests allowed. A negative value
-     * means there is no limit.
-     *
-     * @return the number of Keep-Alive requests that we will allow.
-     */
-    public int getMaxKeepAliveRequests() {
-        return maxKeepAliveRequests;
-    }
-
-
-    /**
-     * Set the maximum size of a POST which will be buffered in SSL mode.
-     * When a POST is received where the security constraints require a client
-     * certificate, the POST body needs to be buffered while an SSL handshake
-     * takes place to obtain the certificate.
-     *
-     * @param msps The maximum size POST body to buffer in bytes
-     */
-    public void setMaxSavePostSize(int msps) {
-        maxSavePostSize = msps;
-    }
-
-
-    /**
-     * Return the maximum size of a POST which will be buffered in SSL mode.
-     *
-     * @return The size in bytes
-     */
-    public int getMaxSavePostSize() {
-        return maxSavePostSize;
-    }
-
-
-    /**
-     * Set the flag to control whether a separate connection timeout is used
-     * during upload of a request body.
-     *
-     * @param isDisabled {@code true} if the separate upload timeout should be
-     *                   disabled
-     */
-    public void setDisableUploadTimeout(boolean isDisabled) {
-        disableUploadTimeout = isDisabled;
-    }
-
-    /**
-     * Get the flag that controls upload time-outs.
-     *
-     * @return {@code true} if the separate upload timeout is disabled
-     */
-    public boolean getDisableUploadTimeout() {
-        return disableUploadTimeout;
-    }
-
-    /**
-     * Set the upload timeout.
-     *
-     * @param timeout Upload timeout in milliseconds
-     */
-    public void setConnectionUploadTimeout(int timeout) {
-        connectionUploadTimeout = timeout ;
-    }
-
-    /**
-     * Get the upload timeout.
-     *
-     * @return Upload timeout in milliseconds
-     */
-    public int getConnectionUploadTimeout() {
-        return connectionUploadTimeout;
-    }
-
-
-    /**
-     * Set the server header name.
-     *
-     * @param server The new value to use for the server header
-     *
-     * @deprecated Use {@link Http11Protocol#setServer(String)}
-     */
-    @Deprecated
-    public void setServer(String server) {
-        protocol.setServer(server);
-    }
-
-
-    @Deprecated
-    public void setServerRemoveAppProvidedValues(boolean serverRemoveAppProvidedValues) {
-        protocol.setServerRemoveAppProvidedValues(serverRemoveAppProvidedValues);
+        return -1;
     }
 
 
@@ -451,7 +239,9 @@ public class Http11Processor extends AbstractProcessor {
      */
     private void addInputFilter(InputFilter[] inputFilters, String encodingName) {
 
-        // Parsing trims and converts to lower case.
+        // Trim provided encoding name and convert to lower case since transfer
+        // encoding names are case insensitive. (RFC2616, section 3.6)
+        encodingName = encodingName.trim().toLowerCase(Locale.ENGLISH);
 
         if (encodingName.equals("identity")) {
             // Skip
@@ -486,6 +276,8 @@ public class Http11Processor extends AbstractProcessor {
 
         // Setting up the I/O
         setSocketWrapper(socketWrapper);
+        inputBuffer.init(socketWrapper);
+        outputBuffer.init(socketWrapper);
 
         // Flags
         keepAlive = true;
@@ -495,11 +287,12 @@ public class Http11Processor extends AbstractProcessor {
         SendfileState sendfileState = SendfileState.DONE;
 
         while (!getErrorState().isError() && keepAlive && !isAsync() && upgradeToken == null &&
-                sendfileState == SendfileState.DONE && !endpoint.isPaused()) {
+                sendfileState == SendfileState.DONE && !protocol.isPaused()) {
 
             // Parsing the request header
             try {
-                if (!inputBuffer.parseRequestLine(keptAlive)) {
+                if (!inputBuffer.parseRequestLine(keptAlive, protocol.getConnectionTimeout(),
+                        protocol.getKeepAliveTimeout())) {
                     if (inputBuffer.getParsingRequestLinePhase() == -1) {
                         return SocketState.UPGRADING;
                     } else if (handleIncompleteRequestLineRead()) {
@@ -507,29 +300,23 @@ public class Http11Processor extends AbstractProcessor {
                     }
                 }
 
-                // Process the Protocol component of the request line
-                // Need to know if this is an HTTP 0.9 request before trying to
-                // parse headers.
-                prepareRequestProtocol();
-
-                if (endpoint.isPaused()) {
+                if (protocol.isPaused()) {
                     // 503 - Service unavailable
                     response.setStatus(503);
                     setErrorState(ErrorState.CLOSE_CLEAN, null);
                 } else {
                     keptAlive = true;
                     // Set this every time in case limit has been changed via JMX
-                    request.getMimeHeaders().setLimit(endpoint.getMaxHeaderCount());
-                    // Don't parse headers for HTTP/0.9
-                    if (!http09 && !inputBuffer.parseHeaders()) {
+                    request.getMimeHeaders().setLimit(protocol.getMaxHeaderCount());
+                    if (!inputBuffer.parseHeaders()) {
                         // We've read part of the request, don't recycle it
                         // instead associate it with the socket
                         openSocket = true;
                         readComplete = false;
                         break;
                     }
-                    if (!disableUploadTimeout) {
-                        socketWrapper.setReadTimeout(connectionUploadTimeout);
+                    if (!protocol.getDisableUploadTimeout()) {
+                        socketWrapper.setReadTimeout(protocol.getConnectionUploadTimeout());
                     }
                 }
             } catch (IOException e) {
@@ -560,15 +347,20 @@ public class Http11Processor extends AbstractProcessor {
             }
 
             // Has an upgrade been requested?
-            if (isConnectionToken(request.getMimeHeaders(), "upgrade")) {
+            Enumeration<String> connectionValues = request.getMimeHeaders().values("Connection");
+            boolean foundUpgrade = false;
+            while (connectionValues.hasMoreElements() && !foundUpgrade) {
+                foundUpgrade = connectionValues.nextElement().toLowerCase(
+                        Locale.ENGLISH).contains("upgrade");
+            }
+
+            if (foundUpgrade) {
                 // Check the protocol
                 String requestedProtocol = request.getHeader("Upgrade");
 
                 UpgradeProtocol upgradeProtocol = protocol.getUpgradeProtocol(requestedProtocol);
                 if (upgradeProtocol != null) {
                     if (upgradeProtocol.accept(request)) {
-                        // TODO Figure out how to handle request bodies at this
-                        // point.
                         response.setStatus(HttpServletResponse.SC_SWITCHING_PROTOCOLS);
                         response.setHeader("Connection", "Upgrade");
                         response.setHeader("Upgrade", requestedProtocol);
@@ -577,7 +369,7 @@ public class Http11Processor extends AbstractProcessor {
 
                         InternalHttpUpgradeHandler upgradeHandler =
                                 upgradeProtocol.getInternalUpgradeHandler(
-                                        getAdapter(), cloneRequest(request));
+                                        socketWrapper, getAdapter(), cloneRequest(request));
                         UpgradeToken upgradeToken = new UpgradeToken(upgradeHandler, null, null);
                         action(ActionCode.UPGRADE, upgradeToken);
                         return SocketState.UPGRADING;
@@ -601,6 +393,7 @@ public class Http11Processor extends AbstractProcessor {
                 }
             }
 
+            int maxKeepAliveRequests = protocol.getMaxKeepAliveRequests();
             if (maxKeepAliveRequests == 1) {
                 keepAlive = false;
             } else if (maxKeepAliveRequests > 0 &&
@@ -670,10 +463,10 @@ public class Http11Processor extends AbstractProcessor {
                 }
             }
 
-            if (!disableUploadTimeout) {
-                int soTimeout = endpoint.getConnectionTimeout();
-                if(soTimeout > 0) {
-                    socketWrapper.setReadTimeout(soTimeout);
+            if (!protocol.getDisableUploadTimeout()) {
+                int connectionTimeout = protocol.getConnectionTimeout();
+                if(connectionTimeout > 0) {
+                    socketWrapper.setReadTimeout(connectionTimeout);
                 } else {
                     socketWrapper.setReadTimeout(0);
                 }
@@ -686,7 +479,7 @@ public class Http11Processor extends AbstractProcessor {
 
         rp.setStage(org.apache.coyote.Constants.STAGE_ENDED);
 
-        if (getErrorState().isError() || (endpoint.isPaused() && !isAsync())) {
+        if (getErrorState().isError() || protocol.isPaused()) {
             return SocketState.CLOSED;
         } else if (isAsync()) {
             return SocketState.LONG;
@@ -710,14 +503,6 @@ public class Http11Processor extends AbstractProcessor {
     }
 
 
-    @Override
-    protected final void setSocketWrapper(SocketWrapperBase<?> socketWrapper) {
-        super.setSocketWrapper(socketWrapper);
-        inputBuffer.init(socketWrapper);
-        outputBuffer.init(socketWrapper);
-    }
-
-
     private Request cloneRequest(Request source) throws IOException {
         Request dest = new Request();
 
@@ -728,7 +513,6 @@ public class Http11Processor extends AbstractProcessor {
         dest.method().duplicate(source.method());
         dest.getMimeHeaders().duplicate(source.getMimeHeaders());
         dest.requestURI().duplicate(source.requestURI());
-        dest.queryString().duplicate(source.queryString());
 
         return dest;
 
@@ -740,7 +524,7 @@ public class Http11Processor extends AbstractProcessor {
         // Check to see if we have read any of the request line yet
         if (inputBuffer.getParsingRequestLinePhase() > 1) {
             // Started to read request line.
-            if (endpoint.isPaused()) {
+            if (protocol.isPaused()) {
                 // Partially processed the request so need to respond
                 response.setStatus(503);
                 setErrorState(ErrorState.CLOSE_CLEAN, null);
@@ -769,15 +553,22 @@ public class Http11Processor extends AbstractProcessor {
     }
 
 
-    private void prepareRequestProtocol() {
+    /**
+     * After reading the request headers, we have to setup the request filters.
+     */
+    private void prepareRequest() {
 
+        http11 = true;
+        http09 = false;
+        contentDelimitation = false;
+
+        if (protocol.isSSLEnabled()) {
+            request.scheme().setString("https");
+        }
         MessageBytes protocolMB = request.protocol();
         if (protocolMB.equals(Constants.HTTP_11)) {
-            http09 = false;
-            http11 = true;
             protocolMB.setString(Constants.HTTP_11);
         } else if (protocolMB.equals(Constants.HTTP_10)) {
-            http09 = false;
             http11 = false;
             keepAlive = false;
             protocolMB.setString(Constants.HTTP_10);
@@ -788,7 +579,6 @@ public class Http11Processor extends AbstractProcessor {
             keepAlive = false;
         } else {
             // Unsupported protocol
-            http09 = false;
             http11 = false;
             // Send 505; Unsupported HTTP version
             response.setStatus(505);
@@ -798,38 +588,25 @@ public class Http11Processor extends AbstractProcessor {
                           " Unsupported HTTP version \""+protocolMB+"\"");
             }
         }
-    }
-
-
-    /**
-     * After reading the request headers, we have to setup the request filters.
-     */
-    private void prepareRequest() throws IOException {
-
-        contentDelimitation = false;
-
-        if (endpoint.isSSLEnabled()) {
-            request.scheme().setString("https");
-        }
 
         MimeHeaders headers = request.getMimeHeaders();
 
         // Check connection header
         MessageBytes connectionValueMB = headers.getValue(Constants.CONNECTION);
-        if (connectionValueMB != null && !connectionValueMB.isNull()) {
-            Set<String> tokens = new HashSet<>();
-            TokenList.parseTokenList(headers.values(Constants.CONNECTION), tokens);
-            if (tokens.contains(Constants.CLOSE)) {
+        if (connectionValueMB != null) {
+            ByteChunk connectionValueBC = connectionValueMB.getByteChunk();
+            if (findBytes(connectionValueBC, Constants.CLOSE_BYTES) != -1) {
                 keepAlive = false;
-            } else if (tokens.contains(Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN)) {
+            } else if (findBytes(connectionValueBC,
+                                 Constants.KEEPALIVE_BYTES) != -1) {
                 keepAlive = true;
             }
         }
 
         if (http11) {
             MessageBytes expectMB = headers.getValue("expect");
-            if (expectMB != null && !expectMB.isNull()) {
-                if (expectMB.toString().trim().equalsIgnoreCase("100-continue")) {
+            if (expectMB != null) {
+                if (expectMB.indexOfIgnoreCase("100-continue", 0) != -1) {
                     inputBuffer.setSwallowInput(false);
                     request.setExpectation(true);
                 } else {
@@ -840,14 +617,14 @@ public class Http11Processor extends AbstractProcessor {
         }
 
         // Check user-agent header
+        Pattern restrictedUserAgents = protocol.getRestrictedUserAgentsPattern();
         if (restrictedUserAgents != null && (http11 || keepAlive)) {
             MessageBytes userAgentValueMB = headers.getValue("user-agent");
             // Check in the restricted list, and adjust the http11
             // and keepAlive flags accordingly
-            if(userAgentValueMB != null && !userAgentValueMB.isNull()) {
+            if(userAgentValueMB != null) {
                 String userAgentValue = userAgentValueMB.toString();
-                if (restrictedUserAgents != null &&
-                        restrictedUserAgents.matcher(userAgentValue).matches()) {
+                if (restrictedUserAgents.matcher(userAgentValue).matches()) {
                     http11 = false;
                     keepAlive = false;
                 }
@@ -861,10 +638,20 @@ public class Http11Processor extends AbstractProcessor {
             hostValueMB = headers.getUniqueValue("host");
         } catch (IllegalArgumentException iae) {
             // Multiple Host headers are not permitted
-            badRequest("http11processor.request.multipleHosts");
+            // 400 - Bad request
+            response.setStatus(400);
+            setErrorState(ErrorState.CLOSE_CLEAN, null);
+            if (log.isDebugEnabled()) {
+                log.debug(sm.getString("http11processor.request.multipleHosts"));
+            }
         }
         if (http11 && hostValueMB == null) {
-            badRequest("http11processor.request.noHostHeader");
+            // 400 - Bad request
+            response.setStatus(400);
+            setErrorState(ErrorState.CLOSE_CLEAN, null);
+            if (log.isDebugEnabled()) {
+                log.debug(sm.getString("http11processor.request.noHostHeader"));
+            }
         }
 
         // Check for an absolute-URI less the query string which has already
@@ -912,7 +699,11 @@ public class Http11Processor extends AbstractProcessor {
                             // Strictly there needs to be a check for valid %nn
                             // encoding here but skip it since it will never be
                             // decoded because the userinfo is ignored
-                            badRequest("http11processor.request.invalidUserInfo");
+                            response.setStatus(400);
+                            setErrorState(ErrorState.CLOSE_CLEAN, null);
+                            if (log.isDebugEnabled()) {
+                                log.debug(sm.getString("http11processor.request.invalidUserInfo"));
+                            }
                             break;
                         }
                     }
@@ -938,26 +729,26 @@ public class Http11Processor extends AbstractProcessor {
                                 // The requirements of RFC 7230 are being
                                 // applied. If the host header and the request
                                 // line do not agree, trigger a 400 response.
-                                badRequest("http11processor.request.inconsistentHosts");
+                                response.setStatus(400);
+                                setErrorState(ErrorState.CLOSE_CLEAN, null);
+                                if (log.isDebugEnabled()) {
+                                    log.debug(sm.getString("http11processor.request.inconsistentHosts"));
+                                }
                             }
                         }
                     }
                 } else {
                     // Not HTTP/1.1 - no Host header so generate one since
                     // Tomcat internals assume it is set
-                    try {
-                        hostValueMB = headers.setValue("host");
-                        hostValueMB.setBytes(uriB, uriBCStart + pos, slashPos - pos);
-                    } catch (IllegalStateException e) {
-                        // Edge case
-                        // If the request has too many headers it won't be
-                        // possible to create the host header. Ignore this as
-                        // processing won't reach the point where the Tomcat
-                        // internals expect there to be a host header.
-                    }
+                    hostValueMB = headers.setValue("host");
+                    hostValueMB.setBytes(uriB, uriBCStart + pos, slashPos - pos);
                 }
             } else {
-                badRequest("http11processor.request.invalidScheme");
+                response.setStatus(400);
+                setErrorState(ErrorState.CLOSE_CLEAN, null);
+                if (log.isDebugEnabled()) {
+                    log.debug(sm.getString("http11processor.request.invalidScheme"));
+                }
             }
         }
 
@@ -965,7 +756,11 @@ public class Http11Processor extends AbstractProcessor {
         // the point of decoding.
         for (int i = uriBC.getStart(); i < uriBC.getEnd(); i++) {
             if (!httpParser.isAbsolutePathRelaxed(uriB[i])) {
-                badRequest("http11processor.request.invalidUri");
+                response.setStatus(400);
+                setErrorState(ErrorState.CLOSE_CLEAN, null);
+                if (log.isDebugEnabled()) {
+                    log.debug(sm.getString("http11processor.request.invalidUri"));
+                }
                 break;
             }
         }
@@ -977,28 +772,24 @@ public class Http11Processor extends AbstractProcessor {
         if (http11) {
             MessageBytes transferEncodingValueMB = headers.getValue("transfer-encoding");
             if (transferEncodingValueMB != null) {
-                List<String> encodingNames = new ArrayList<>();
-                if (TokenList.parseTokenList(headers.values("transfer-encoding"), encodingNames)) {
-                    for (String encodingName : encodingNames) {
-                        // "identity" codings are ignored
-                        addInputFilter(inputFilters, encodingName);
-                    }
-                } else {
-                    // Invalid transfer encoding
-                    badRequest("http11processor.request.invalidTransferEncoding");
+                String transferEncodingValue = transferEncodingValueMB.toString();
+                // Parse the comma separated list. "identity" codings are ignored
+                int startPos = 0;
+                int commaPos = transferEncodingValue.indexOf(',');
+                String encodingName = null;
+                while (commaPos != -1) {
+                    encodingName = transferEncodingValue.substring(startPos, commaPos);
+                    addInputFilter(inputFilters, encodingName);
+                    startPos = commaPos + 1;
+                    commaPos = transferEncodingValue.indexOf(',', startPos);
                 }
+                encodingName = transferEncodingValue.substring(startPos);
+                addInputFilter(inputFilters, encodingName);
             }
         }
 
         // Parse content-length header
-        long contentLength = -1;
-        try {
-            contentLength = request.getContentLengthLong();
-        } catch (NumberFormatException e) {
-            badRequest("http11processor.request.nonNumericContentLength");
-        } catch (IllegalArgumentException e) {
-            badRequest("http11processor.request.multipleContentLength");
-        }
+        long contentLength = request.getContentLengthLong();
         if (contentLength >= 0) {
             if (contentDelimitation) {
                 // contentDelimitation being true at this point indicates that
@@ -1031,15 +822,6 @@ public class Http11Processor extends AbstractProcessor {
     }
 
 
-    private void badRequest(String errorKey) {
-        response.setStatus(400);
-        setErrorState(ErrorState.CLOSE_CLEAN, null);
-        if (log.isDebugEnabled()) {
-            log.debug(sm.getString(errorKey));
-        }
-    }
-
-
     /**
      * When committing the response, we have to validate the set of headers, as
      * well as setup the response filters.
@@ -1085,11 +867,12 @@ public class Http11Processor extends AbstractProcessor {
         }
 
         // Sendfile support
-        if (endpoint.getUseSendfile()) {
+        if (protocol.getUseSendfile()) {
             prepareSendfile(outputFilters);
         }
 
         // Check for compression
+
         boolean useCompression = false;
         if (entityBody && sendfileData == null) {
             useCompression = protocol.useCompression(request, response);
@@ -1110,14 +893,20 @@ public class Http11Processor extends AbstractProcessor {
         }
 
         long contentLength = response.getContentLengthLong();
-        boolean connectionClosePresent = isConnectionToken(headers, Constants.CLOSE);
-        if (contentLength != -1) {
+        boolean connectionClosePresent = false;
+        if (http11 && response.getTrailerFields() != null) {
+            // If trailer fields are set, always use chunking
+            outputBuffer.addActiveFilter(outputFilters[Constants.CHUNKED_FILTER]);
+            contentDelimitation = true;
+            headers.addValue(Constants.TRANSFERENCODING).setString(Constants.CHUNKED);
+        } else if (contentLength != -1) {
             headers.setValue("Content-Length").setLong(contentLength);
             outputBuffer.addActiveFilter(outputFilters[Constants.IDENTITY_FILTER]);
             contentDelimitation = true;
         } else {
             // If the response code supports an entity body and we're on
             // HTTP 1.1 then we chunk unless we have a Connection: close header
+            connectionClosePresent = isConnectionClose(headers);
             if (http11 && entityBody && !connectionClosePresent) {
                 outputBuffer.addActiveFilter(outputFilters[Constants.CHUNKED_FILTER]);
                 contentDelimitation = true;
@@ -1161,36 +950,8 @@ public class Http11Processor extends AbstractProcessor {
                 headers.addValue(Constants.CONNECTION).setString(
                         Constants.CLOSE);
             }
-        } else if (!getErrorState().isError()) {
-            if (!http11) {
-                headers.addValue(Constants.CONNECTION).setString(Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
-            }
-
-            if (protocol.getUseKeepAliveResponseHeader()) {
-                boolean connectionKeepAlivePresent =
-                    isConnectionToken(request.getMimeHeaders(), Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
-
-                if (connectionKeepAlivePresent) {
-                    int keepAliveTimeout = protocol.getKeepAliveTimeout();
-
-                    if (keepAliveTimeout > 0) {
-                        String value = "timeout=" + keepAliveTimeout / 1000L;
-                        headers.setValue(Constants.KEEP_ALIVE_HEADER_NAME).setString(value);
-
-                        if (http11) {
-                            // Append if there is already a Connection header,
-                            // else create the header
-                            MessageBytes connectionHeaderValue = headers.getValue(Constants.CONNECTION);
-                            if (connectionHeaderValue == null) {
-                                headers.addValue(Constants.CONNECTION).setString(Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
-                            } else {
-                                connectionHeaderValue.setString(
-                                        connectionHeaderValue.getString() + ", " + Constants.KEEP_ALIVE_HEADER_VALUE_TOKEN);
-                            }
-                        }
-                    }
-                }
-            }
+        } else if (!http11 && !getErrorState().isError()) {
+            headers.addValue(Constants.CONNECTION).setString(Constants.KEEPALIVE);
         }
 
         // Add server header
@@ -1224,18 +985,14 @@ public class Http11Processor extends AbstractProcessor {
         outputBuffer.commit();
     }
 
-    private static boolean isConnectionToken(MimeHeaders headers, String token) throws IOException {
+    private static boolean isConnectionClose(MimeHeaders headers) {
         MessageBytes connection = headers.getValue(Constants.CONNECTION);
         if (connection == null) {
             return false;
         }
-
-        Set<String> tokens = new HashSet<>();
-        TokenList.parseTokenList(headers.values(Constants.CONNECTION), tokens);
-        return tokens.contains(token);
+        return connection.equals(Constants.CLOSE);
     }
 
-
     private void prepareSendfile(OutputFilter[] outputFilters) {
         String fileName = (String) request.getAttribute(
                 org.apache.coyote.Constants.SENDFILE_FILENAME_ATTR);
@@ -1295,7 +1052,7 @@ public class Http11Processor extends AbstractProcessor {
 
     @Override
     protected SocketState dispatchEndRequest() {
-        if (!keepAlive || endpoint.isPaused()) {
+        if (!keepAlive) {
             return SocketState.CLOSED;
         } else {
             endRequest();
@@ -1426,7 +1183,7 @@ public class Http11Processor extends AbstractProcessor {
             // interfere with the client's handshake messages
             InputFilter[] inputFilters = inputBuffer.getFilters();
             ((BufferedInputFilter) inputFilters[Constants.BUFFERED_FILTER]).setLimit(
-                    maxSavePostSize);
+                    protocol.getMaxSavePostSize());
             inputBuffer.addActiveFilter(inputFilters[Constants.BUFFERED_FILTER]);
 
             /*
@@ -1464,7 +1221,7 @@ public class Http11Processor extends AbstractProcessor {
 
 
     @Override
-    protected final boolean isReadyForWrite() {
+    protected final boolean isReady() {
         return outputBuffer.isReady();
     }
 
@@ -1495,6 +1252,34 @@ public class Http11Processor extends AbstractProcessor {
     }
 
 
+    @Override
+    protected boolean isTrailerFieldsReady() {
+        if (inputBuffer.isChunking()) {
+            return inputBuffer.isFinished();
+        } else {
+            return true;
+        }
+    }
+
+
+    @Override
+    protected boolean isTrailerFieldsSupported() {
+        // Request must be HTTP/1.1 to support trailer fields
+        if (!http11) {
+            return false;
+        }
+
+        // If the response is not yet committed, chunked encoding can be used
+        // and the trailer fields sent
+        if (!response.isCommitted()) {
+            return true;
+        }
+
+        // Response has been committed - need to see if chunked is being used
+        return outputBuffer.isChunking();
+    }
+
+
     /**
      * Trigger sendfile processing if required.
      *
@@ -1541,6 +1326,7 @@ public class Http11Processor extends AbstractProcessor {
         upgradeToken = null;
         socketWrapper = null;
         sendfileData = null;
+        sslSupport = null;
     }
 
 
