@@ -119,6 +119,12 @@ public class DigesterFactory {
         addSelf(systemIds, "javaee_web_services_1_4.xsd");
         addSelf(systemIds, "javaee_web_services_client_1_4.xsd");
 
+        // from JavaEE 8
+        add(systemIds, XmlIdentifiers.WEB_40_XSD, locationFor("web-app_4_0.xsd"));
+        add(systemIds, XmlIdentifiers.WEB_FRAGMENT_40_XSD, locationFor("web-fragment_4_0.xsd"));
+        addSelf(systemIds, "web-common_4_0.xsd");
+        addSelf(systemIds, "javaee_8.xsd");
+
         SERVLET_API_PUBLIC_IDS = Collections.unmodifiableMap(publicIds);
         SERVLET_API_SYSTEM_IDS = Collections.unmodifiableMap(systemIds);
     }
@@ -134,13 +140,6 @@ public class DigesterFactory {
     private static void add(Map<String,String> ids, String id, String location) {
         if (location != null) {
             ids.put(id, location);
-            // BZ 63311
-            // Support http and https locations as the move away from http and
-            // towards https continues.
-            if (id.startsWith("http://")) {
-                String httpsId = "https://" + id.substring(7);
-                ids.put(httpsId, location);
-            }
         }
     }
 
