@@ -41,7 +41,7 @@ import org.apache.tomcat.util.ExceptionUtils;
 import org.apache.tomcat.util.IntrospectionUtils;
 import org.apache.tomcat.util.collections.SynchronizedStack;
 import org.apache.tomcat.util.modeler.Registry;
-import org.apache.tomcat.util.net.AbstractEndpoint.Acceptor.AcceptorState;
+import org.apache.tomcat.util.net.Acceptor.AcceptorState;
 import org.apache.tomcat.util.res.StringManager;
 import org.apache.tomcat.util.threads.LimitLatch;
 import org.apache.tomcat.util.threads.ResizableExecutor;
@@ -50,12 +50,15 @@ import org.apache.tomcat.util.threads.TaskThreadFactory;
 import org.apache.tomcat.util.threads.ThreadPoolExecutor;
 
 /**
- * @param <S> The type for the sockets managed by this endpoint.
+ * @param <S> The type used by the socket wrapper associated with this endpoint.
+ *            May be the same as U.
+ * @param <U> The type of the underlying socket used by this endpoint. May be
+ *            the same as S.
  *
  * @author Mladen Turk
  * @author Remy Maucherat
  */
-public abstract class AbstractEndpoint<S> {
+public abstract class AbstractEndpoint<S,U> {
 
     // -------------------------------------------------------------- Constants
 
@@ -128,34 +131,6 @@ public abstract class AbstractEndpoint<S> {
         UNBOUND, BOUND_ON_INIT, BOUND_ON_START, SOCKET_CLOSED_ON_STOP
     }
 
-    public abstract static class Acceptor implements Runnable {
-        public enum AcceptorState {
-            NEW, RUNNING, PAUSED, ENDED
-        }
-
-        protected volatile AcceptorState state = AcceptorState.NEW;
-        public final AcceptorState getState() {
-            return state;
-        }
-
-        private String threadName;
-        protected final void setThreadName(final String threadName) {
-            this.threadName = threadName;
-        }
-        protected final String getThreadName() {
-            return threadName;
-        }
-    }
-
-
-    private static final int INITIAL_ERROR_DELAY = 50;
-    private static final int MAX_ERROR_DELAY = 1600;
-
-
-    public static long toTimeout(long timeout) {
-        // Many calls can't do infinite timeout so use Long.MAX_VALUE if timeout is <= 0
-        return (timeout > 0) ? timeout : Long.MAX_VALUE;
-    }
 
     // ----------------------------------------------------------------- Fields
 
@@ -184,7 +159,7 @@ public abstract class AbstractEndpoint<S> {
     /**
      * Socket properties
      */
-    protected SocketProperties socketProperties = new SocketProperties();
+    protected final SocketProperties socketProperties = new SocketProperties();
     public SocketProperties getSocketProperties() {
         return socketProperties;
     }
@@ -192,7 +167,7 @@ public abstract class AbstractEndpoint<S> {
     /**
      * Threads used to accept new connections and pass them to worker threads.
      */
-    protected Acceptor[] acceptors;
+    protected List<Acceptor<U>> acceptors;
 
     /**
      * Cache for SocketProcessor objects
@@ -246,6 +221,7 @@ public abstract class AbstractEndpoint<S> {
         }
         if (bindState != BindState.UNBOUND && bindState != BindState.SOCKET_CLOSED_ON_STOP &&
                 isSSLEnabled()) {
+            sslHostConfig.setConfigType(getSslConfigType());
             try {
                 createSSLContext(sslHostConfig);
             } catch (Exception e) {
@@ -324,6 +300,8 @@ public abstract class AbstractEndpoint<S> {
         return sslHostConfigs.values().toArray(new SSLHostConfig[0]);
     }
 
+    protected abstract SSLHostConfig.Type getSslConfigType();
+
     /**
      * Create the SSLContextfor the the given SSLHostConfig.
      *
@@ -334,34 +312,13 @@ public abstract class AbstractEndpoint<S> {
      */
     protected abstract void createSSLContext(SSLHostConfig sslHostConfig) throws Exception;
 
-
-    protected void destroySsl() throws Exception {
-        if (isSSLEnabled()) {
-            for (SSLHostConfig sslHostConfig : sslHostConfigs.values()) {
-                releaseSSLContext(sslHostConfig);
-            }
-        }
-    }
-
-
     /**
      * Release the SSLContext, if any, associated with the SSLHostConfig.
      *
      * @param sslHostConfig The SSLHostConfig for which the SSLContext should be
      *                      released
      */
-    protected void releaseSSLContext(SSLHostConfig sslHostConfig) {
-        for (SSLHostConfigCertificate certificate : sslHostConfig.getCertificates(true)) {
-            if (certificate.getSslContext() != null) {
-                SSLContext sslContext = certificate.getSslContext();
-                if (sslContext != null) {
-                    sslContext.destroy();
-                }
-            }
-        }
-    }
-
-
+    protected abstract void releaseSSLContext(SSLHostConfig sslHostConfig);
 
     protected SSLHostConfig getSSLHostConfig(String sniHostName) {
         SSLHostConfig result = null;
@@ -494,11 +451,32 @@ public abstract class AbstractEndpoint<S> {
     /**
      * Server socket port.
      */
-    private int port;
+    private int port = -1;
     public int getPort() { return port; }
     public void setPort(int port ) { this.port=port; }
 
 
+    private int portOffset = 0;
+    public int getPortOffset() { return portOffset; }
+    public void setPortOffset(int portOffset ) {
+        if (portOffset < 0) {
+            throw new IllegalArgumentException(
+                    sm.getString("endpoint.portOffset.invalid", Integer.valueOf(portOffset)));
+        }
+        this.portOffset = portOffset;
+    }
+
+
+    public int getPortWithOffset() {
+        // Zero is a special case and negative values are invalid
+        int port = getPort();
+        if (port > 0) {
+            return port + getPortOffset();
+        }
+        return port;
+    }
+
+
     public final int getLocalPort() {
         try {
             InetSocketAddress localAddress = getLocalAddress();
@@ -543,10 +521,6 @@ public abstract class AbstractEndpoint<S> {
     private int acceptCount = 100;
     public void setAcceptCount(int acceptCount) { if (acceptCount > 0) this.acceptCount = acceptCount; }
     public int getAcceptCount() { return acceptCount; }
-    @Deprecated
-    public void setBacklog(int backlog) { setAcceptCount(backlog); }
-    @Deprecated
-    public int getBacklog() { return getAcceptCount(); }
 
     /**
      * Controls when the Endpoint binds the port. <code>true</code>, the default
@@ -596,10 +570,6 @@ public abstract class AbstractEndpoint<S> {
         socketProperties.setSoLingerTime(connectionLinger);
         socketProperties.setSoLingerOn(connectionLinger>=0);
     }
-    @Deprecated
-    public int getSoLinger() { return getConnectionLinger(); }
-    @Deprecated
-    public void setSoLinger(int soLinger) { setConnectionLinger(soLinger);}
 
 
     /**
@@ -609,10 +579,6 @@ public abstract class AbstractEndpoint<S> {
      */
     public int getConnectionTimeout() { return socketProperties.getSoTimeout(); }
     public void setConnectionTimeout(int soTimeout) { socketProperties.setSoTimeout(soTimeout); }
-    @Deprecated
-    public int getSoTimeout() { return getConnectionTimeout(); }
-    @Deprecated
-    public void setSoTimeout(int soTimeout) { setConnectionTimeout(soTimeout); }
 
     /**
      * SSL engine.
@@ -707,17 +673,6 @@ public abstract class AbstractEndpoint<S> {
         this.maxKeepAliveRequests = maxKeepAliveRequests;
     }
 
-    /**
-     * The maximum number of headers in a request that are allowed.
-     * 100 by default. A value of less than 0 means no limit.
-     */
-    private int maxHeaderCount = 100; // as in Apache HTTPD server
-    public int getMaxHeaderCount() {
-        return maxHeaderCount;
-    }
-    public void setMaxHeaderCount(int maxHeaderCount) {
-        this.maxHeaderCount = maxHeaderCount;
-    }
 
     /**
      * Name of the thread pool, which will be used for naming child threads.
@@ -745,14 +700,6 @@ public abstract class AbstractEndpoint<S> {
     public boolean getDaemon() { return daemon; }
 
 
-    /**
-     * Expose asynchronous IO capability.
-     */
-    private boolean useAsyncIO = true;
-    public void setUseAsyncIO(boolean useAsyncIO) { this.useAsyncIO = useAsyncIO; }
-    public boolean getUseAsyncIO() { return useAsyncIO; }
-
-
     protected abstract boolean getDeferAccept();
 
 
@@ -923,12 +870,12 @@ public abstract class AbstractEndpoint<S> {
     }
 
     /**
-     * Unlock the server socket accept using a bogus connection.
+     * Unlock the server socket acceptor threads using bogus connections.
      */
-    protected void unlockAccept() {
+    private void unlockAccept() {
         // Only try to unlock the acceptor if it is necessary
         int unlocksRequired = 0;
-        for (Acceptor acceptor : acceptors) {
+        for (Acceptor<U> acceptor : acceptors) {
             if (acceptor.getState() == AcceptorState.RUNNING) {
                 unlocksRequired++;
             }
@@ -986,7 +933,7 @@ public abstract class AbstractEndpoint<S> {
             }
             // Wait for upto 1000ms acceptor threads to unlock
             long waitLeft = 1000;
-            for (Acceptor acceptor : acceptors) {
+            for (Acceptor<U> acceptor : acceptors) {
                 while (waitLeft > 0 &&
                         acceptor.getState() == AcceptorState.RUNNING) {
                     Thread.sleep(5);
@@ -996,7 +943,8 @@ public abstract class AbstractEndpoint<S> {
         } catch(Throwable t) {
             ExceptionUtils.handleThrowable(t);
             if (getLog().isDebugEnabled()) {
-                getLog().debug(sm.getString("endpoint.debug.unlock.fail", "" + getPort()), t);
+                getLog().debug(sm.getString(
+                        "endpoint.debug.unlock.fail", String.valueOf(getPortWithOffset())), t);
             }
         }
     }
@@ -1113,9 +1061,23 @@ public abstract class AbstractEndpoint<S> {
     public abstract void startInternal() throws Exception;
     public abstract void stopInternal() throws Exception;
 
-    public void init() throws Exception {
-        if (bindOnInit) {
+
+    private void bindWithCleanup() throws Exception {
+        try {
             bind();
+        } catch (Throwable t) {
+            // Ensure open sockets etc. are cleaned up if something goes
+            // wrong during bind
+            ExceptionUtils.handleThrowable(t);
+            unbind();
+            throw t;
+        }
+    }
+
+
+    public final void init() throws Exception {
+        if (bindOnInit) {
+            bindWithCleanup();
             bindState = BindState.BOUND_ON_INIT;
         }
         if (this.domain != null) {
@@ -1136,10 +1098,6 @@ public abstract class AbstractEndpoint<S> {
 
 
     private void registerJmx(SSLHostConfig sslHostConfig) {
-        if (domain == null) {
-            // Before init the domain is null
-            return;
-        }
         ObjectName sslOname = null;
         try {
             sslOname = new ObjectName(domain + ":type=SSLHostConfig,ThreadPool=\"" +
@@ -1188,21 +1146,23 @@ public abstract class AbstractEndpoint<S> {
 
     public final void start() throws Exception {
         if (bindState == BindState.UNBOUND) {
-            bind();
+            bindWithCleanup();
             bindState = BindState.BOUND_ON_START;
         }
         startInternal();
     }
 
+
     protected final void startAcceptorThreads() {
         int count = getAcceptorThreadCount();
-        acceptors = new Acceptor[count];
+        acceptors = new ArrayList<>(count);
 
         for (int i = 0; i < count; i++) {
-            acceptors[i] = createAcceptor();
+            Acceptor<U> acceptor = new Acceptor<>(this);
             String threadName = getName() + "-Acceptor-" + i;
-            acceptors[i].setThreadName(threadName);
-            Thread t = new Thread(acceptors[i], threadName);
+            acceptor.setThreadName(threadName);
+            acceptors.add(acceptor);
+            Thread t = new Thread(acceptor, threadName);
             t.setPriority(getAcceptorThreadPriority());
             t.setDaemon(getDaemon());
             t.start();
@@ -1211,18 +1171,13 @@ public abstract class AbstractEndpoint<S> {
 
 
     /**
-     * Hook to allow Endpoints to provide a specific Acceptor implementation.
-     * @return the acceptor
-     */
-    protected abstract Acceptor createAcceptor();
-
-
-    /**
-     * Pause the endpoint, which will stop it accepting new connections.
+     * Pause the endpoint, which will stop it accepting new connections and
+     * unlock the acceptor.
      */
     public void pause() {
         if (running && !paused) {
             paused = true;
+            releaseConnectionLatch();
             unlockAccept();
             getHandler().pause();
         }
@@ -1270,7 +1225,7 @@ public abstract class AbstractEndpoint<S> {
         return connectionLimitLatch;
     }
 
-    protected void releaseConnectionLatch() {
+    private void releaseConnectionLatch() {
         LimitLatch latch = connectionLimitLatch;
         if (latch!=null) latch.releaseAll();
         connectionLimitLatch = null;
@@ -1294,37 +1249,6 @@ public abstract class AbstractEndpoint<S> {
         } else return -1;
     }
 
-    /**
-     * Provides a common approach for sub-classes to handle exceptions where a
-     * delay is required to prevent a Thread from entering a tight loop which
-     * will consume CPU and may also trigger large amounts of logging. For
-     * example, this can happen with the Acceptor thread if the ulimit for open
-     * files is reached.
-     *
-     * @param currentErrorDelay The current delay being applied on failure
-     * @return  The delay to apply on the next failure
-     */
-    protected int handleExceptionWithDelay(int currentErrorDelay) {
-        // Don't delay on first exception
-        if (currentErrorDelay > 0) {
-            try {
-                Thread.sleep(currentErrorDelay);
-            } catch (InterruptedException e) {
-                // Ignore
-            }
-        }
-
-        // On subsequent exceptions, start the delay at 50ms, doubling the delay
-        // on every subsequent exception until the delay reaches 1.6 seconds.
-        if (currentErrorDelay == 0) {
-            return INITIAL_ERROR_DELAY;
-        } else if (currentErrorDelay < MAX_ERROR_DELAY) {
-            return currentErrorDelay * 2;
-        } else {
-            return MAX_ERROR_DELAY;
-        }
-    }
-
 
     /**
      * Close the server socket (to prevent further connections) if the server
@@ -1352,5 +1276,14 @@ public abstract class AbstractEndpoint<S> {
      */
     protected abstract void doCloseServerSocket() throws IOException;
 
+    protected abstract U serverSocketAccept() throws Exception;
+
+    protected abstract boolean setSocketOptions(U socket);
+
+    protected abstract void closeSocket(U socket);
+
+    protected void destroySocket(U socket) {
+        closeSocket(socket);
+    }
 }
 
