@@ -75,12 +75,6 @@ public class JarFileUrlJar implements Jar {
     }
 
 
-    @Override
-    @Deprecated
-    public boolean entryExists(String name) {
-        return false;
-    }
-
     @Override
     public InputStream getInputStream(String name) throws IOException {
         // JarFile#getEntry() is multi-release aware
@@ -103,13 +97,6 @@ public class JarFileUrlJar implements Jar {
         }
     }
 
-    @Override
-    public boolean exists(String name) throws IOException {
-        // JarFile#getEntry() is multi-release aware
-        ZipEntry entry = jarFile.getEntry(name);
-        return entry != null;
-    }
-
     @Override
     public String getURL(String entry) {
         StringBuilder result = new StringBuilder("jar:");
