@@ -28,7 +28,6 @@ import java.nio.charset.StandardCharsets;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Locale;
-import java.util.Map;
 import java.util.Random;
 
 import javax.net.SocketFactory;
@@ -50,9 +49,8 @@ import org.apache.coyote.http2.HpackDecoder.HeaderEmitter;
 import org.apache.coyote.http2.Http2Parser.Input;
 import org.apache.coyote.http2.Http2Parser.Output;
 import org.apache.tomcat.util.codec.binary.Base64;
-import org.apache.tomcat.util.http.FastHttpDateFormat;
+import org.apache.tomcat.util.compat.JrePlatform;
 import org.apache.tomcat.util.http.MimeHeaders;
-import org.apache.tomcat.util.net.TesterSupport;
 
 /**
  * Tests for compliance with the <a href="https://tools.ietf.org/html/rfc7540">
@@ -64,7 +62,6 @@ public abstract class Http2TestBase extends TomcatBaseTest {
     // test that demonstrated that most HTTP/2 tests were failing because the
     // response now included a date header
     protected static final String DEFAULT_DATE = "Wed, 11 Nov 2015 19:18:42 GMT";
-    protected static final long DEFAULT_TIME = FastHttpDateFormat.parseDate(DEFAULT_DATE);
 
     private static final String HEADER_IGNORED = "x-ignore";
 
@@ -75,13 +72,12 @@ public abstract class Http2TestBase extends TomcatBaseTest {
 
     static {
         byte[] empty = new byte[0];
-        EMPTY_HTTP2_SETTINGS_HEADER = "HTTP2-Settings: " + Base64.encodeBase64URLSafeString(empty) + "\r\n";
+        EMPTY_HTTP2_SETTINGS_HEADER = "HTTP2-Settings: " + Base64.encodeBase64String(empty) + "\r\n";
     }
 
-    protected static final String TRAILER_HEADER_NAME = "X-TrailerTest";
+    protected static final String TRAILER_HEADER_NAME = "x-trailertest";
     protected static final String TRAILER_HEADER_VALUE = "test";
 
-    // Client
     private Socket s;
     protected HpackEncoder hpackEncoder;
     protected Input input;
@@ -89,9 +85,6 @@ public abstract class Http2TestBase extends TomcatBaseTest {
     protected Http2Parser parser;
     protected OutputStream os;
 
-    // Server
-    protected Http2Protocol http2Protocol;
-
     private long pingAckDelayMillis = 0;
 
 
@@ -104,13 +97,9 @@ public abstract class Http2TestBase extends TomcatBaseTest {
      * that the first response is correctly received.
      */
     protected void http2Connect() throws Exception {
-        http2Connect(false);
-    }
-
-    protected void http2Connect(boolean tls) throws Exception {
-        enableHttp2(tls);
+        enableHttp2();
         configureAndStartWebApplication();
-        openClientConnection(tls);
+        openClientConnection();
         doHttpUpgrade();
         sendClientPreface();
         validateHttp2InitialResponse();
@@ -313,24 +302,6 @@ public abstract class Http2TestBase extends TomcatBaseTest {
     }
 
 
-    protected void sendParameterPostRequest(int streamId, byte[] padding, String body,
-            long contentLength, boolean useExpectation) throws IOException {
-        byte[] headersFrameHeader = new byte[9];
-        ByteBuffer headersPayload = ByteBuffer.allocate(128);
-        byte[] dataFrameHeader = new byte[9];
-        ByteBuffer dataPayload = ByteBuffer.allocate(128);
-
-        buildPostRequest(headersFrameHeader, headersPayload, useExpectation,
-                "application/x-www-form-urlencoded", contentLength, "/parameter", dataFrameHeader,
-                dataPayload, padding, null, null, streamId);
-        writeFrame(headersFrameHeader, headersPayload);
-        if (body != null) {
-            dataPayload.put(body.getBytes(StandardCharsets.ISO_8859_1));
-            writeFrame(dataFrameHeader, dataPayload);
-        }
-    }
-
-
     protected void buildPostRequest(byte[] headersFrameHeader, ByteBuffer headersPayload,
             boolean useExpectation, byte[] dataFrameHeader, ByteBuffer dataPayload, byte[] padding,
             int streamId) {
@@ -341,29 +312,14 @@ public abstract class Http2TestBase extends TomcatBaseTest {
     protected void buildPostRequest(byte[] headersFrameHeader, ByteBuffer headersPayload,
             boolean useExpectation, byte[] dataFrameHeader, ByteBuffer dataPayload, byte[] padding,
             byte[] trailersFrameHeader, ByteBuffer trailersPayload, int streamId) {
-        buildPostRequest(headersFrameHeader, headersPayload, useExpectation, null, -1, "/simple",
-                dataFrameHeader, dataPayload, padding, trailersFrameHeader, trailersPayload, streamId);
-    }
-
-    protected void buildPostRequest(byte[] headersFrameHeader, ByteBuffer headersPayload,
-            boolean useExpectation, String contentType, long contentLength, String path,
-            byte[] dataFrameHeader, ByteBuffer dataPayload, byte[] padding,
-            byte[] trailersFrameHeader, ByteBuffer trailersPayload, int streamId) {
-
         MimeHeaders headers = new MimeHeaders();
         headers.addValue(":method").setString("POST");
         headers.addValue(":scheme").setString("http");
-        headers.addValue(":path").setString(path);
+        headers.addValue(":path").setString("/simple");
         headers.addValue(":authority").setString("localhost:" + getPort());
         if (useExpectation) {
             headers.addValue("expect").setString("100-continue");
         }
-        if (contentType != null) {
-            headers.addValue("content-type").setString(contentType);
-        }
-        if (contentLength > -1) {
-            headers.addValue("content-length").setLong(contentLength);
-        }
         hpackEncoder.encode(headers, headersPayload);
 
         headersPayload.flip();
@@ -486,41 +442,25 @@ public abstract class Http2TestBase extends TomcatBaseTest {
 
 
     protected String getCookieResponseTrace(int streamId, int cookieCount) {
-        return getResponseBodyFrameTrace(streamId, 200, "text/plain;charset=UTF-8", null,
-                "Cookie count: " + cookieCount, null);
+        return getResponseBodyFrameTrace(streamId, "text/plain;charset=UTF-8",
+                "Cookie count: " + cookieCount);
     }
 
 
-    protected String getResponseBodyFrameTrace(int streamId, String body) {
-        return getResponseBodyFrameTrace(streamId, 200, "application/octet-stream", null, body, body);
+    private String getResponseBodyFrameTrace(int streamId, String body) {
+        return getResponseBodyFrameTrace(streamId, "application/octet-stream", body);
     }
 
-
-    protected String getResponseBodyFrameTrace(int streamId, int status, String contentType,
-            String contentLanguage, String body, String cl) {
+    private String getResponseBodyFrameTrace(int streamId, String contentType, String body) {
         StringBuilder result = new StringBuilder();
         result.append(streamId);
         result.append("-HeadersStart\n");
         result.append(streamId);
-        result.append("-Header-[:status]-[");
-        result.append(status);
-        result.append("]\n");
+        result.append("-Header-[:status]-[200]\n");
         result.append(streamId);
         result.append("-Header-[content-type]-[");
         result.append(contentType);
         result.append("]\n");
-        if (contentLanguage != null) {
-            result.append(streamId);
-            result.append("-Header-[content-language]-[");
-            result.append(contentLanguage);
-            result.append("]\n");
-        }
-        if (cl != null) {
-            result.append(streamId);
-            result.append("-Header-[content-length]-[");
-            result.append(cl);
-            result.append("]\n");
-        }
         result.append(streamId);
         result.append("-Header-[date]-[");
         result.append(DEFAULT_DATE);
@@ -543,37 +483,16 @@ public abstract class Http2TestBase extends TomcatBaseTest {
     }
 
     protected void enableHttp2(long maxConcurrentStreams) {
-        enableHttp2(maxConcurrentStreams, false);
-    }
-
-    protected void enableHttp2(boolean tls) {
-        enableHttp2(200, tls);
-    }
-
-    protected void enableHttp2(long maxConcurrentStreams, boolean tls) {
-        Tomcat tomcat = getTomcatInstance();
-        Connector connector = tomcat.getConnector();
-        http2Protocol = new UpgradableHttp2Protocol();
+        Connector connector = getTomcatInstance().getConnector();
+        Http2Protocol http2Protocol = new Http2Protocol();
         // Short timeouts for now. May need to increase these for CI systems.
-        http2Protocol.setReadTimeout(6000);
-        http2Protocol.setWriteTimeout(6000);
-        http2Protocol.setKeepAliveTimeout(15000);
-        http2Protocol.setStreamReadTimeout(3000);
-        http2Protocol.setStreamWriteTimeout(3000);
+        http2Protocol.setReadTimeout(2000);
+        http2Protocol.setKeepAliveTimeout(5000);
+        http2Protocol.setWriteTimeout(2000);
         http2Protocol.setMaxConcurrentStreams(maxConcurrentStreams);
         connector.addUpgradeProtocol(http2Protocol);
-        if (tls) {
-            // Enable TLS
-            TesterSupport.initSsl(tomcat);
-        }
     }
 
-    private class UpgradableHttp2Protocol extends Http2Protocol {
-        @Override
-        public String getHttpUpgradeName(boolean isSSLEnabled) {
-            return "h2c";
-        }
-    }
 
     protected void configureAndStartWebApplication() throws LifecycleException {
         Tomcat tomcat = getTomcatInstance();
@@ -587,21 +506,14 @@ public abstract class Http2TestBase extends TomcatBaseTest {
         ctxt.addServletMappingDecoded("/large", "large");
         Tomcat.addServlet(ctxt, "cookie", new CookieServlet());
         ctxt.addServletMappingDecoded("/cookie", "cookie");
-        Tomcat.addServlet(ctxt, "parameter", new ParameterServlet());
-        ctxt.addServletMappingDecoded("/parameter", "parameter");
 
         tomcat.start();
     }
 
 
     protected void openClientConnection() throws IOException {
-        openClientConnection(false);
-    }
-
-    protected void openClientConnection(boolean tls) throws IOException {
-        SocketFactory socketFactory = tls ? TesterSupport.configureClientSsl() : SocketFactory.getDefault();
         // Open a connection
-        s = socketFactory.createSocket("localhost", getPort());
+        s = SocketFactory.getDefault().createSocket("localhost", getPort());
         s.setSoTimeout(30000);
 
         os = s.getOutputStream();
@@ -777,7 +689,7 @@ public abstract class Http2TestBase extends TomcatBaseTest {
         pingHeader[3] = FrameType.PING.getIdByte();
         // Flags
         if (ack) {
-            ByteUtil.setOneBytes(pingHeader, 4, 0x01);
+            setOneBytes(pingHeader, 4, 0x01);
         }
         // Stream
         ByteUtil.set31Bits(pingHeader, 5, streamId);
@@ -854,7 +766,7 @@ public abstract class Http2TestBase extends TomcatBaseTest {
 
         // Payload
         ByteUtil.set31Bits(priorityFrame, 9, streamDependencyId);
-        ByteUtil.setOneBytes(priorityFrame, 13, weight);
+        setOneBytes(priorityFrame, 13, weight);
 
         os.write(priorityFrame);
         os.flush();
@@ -927,11 +839,16 @@ public abstract class Http2TestBase extends TomcatBaseTest {
                     connector.getProtocolHandlerClassName().contains("Nio2"));
 
             Assume.assumeTrue("This test is only expected to trigger an exception on Windows",
-                    System.getProperty("os.name").startsWith("Windows"));
+                    JrePlatform.IS_WINDOWS);
         }
     }
 
 
+    static void setOneBytes(byte[] output, int firstByte, int value) {
+        output[firstByte] = (byte) (value & 0xFF);
+    }
+
+
     private static class TestInput implements Http2Parser.Input {
 
         private final InputStream is;
@@ -942,21 +859,6 @@ public abstract class Http2TestBase extends TomcatBaseTest {
         }
 
 
-        @Override
-        public boolean fill(boolean block, byte[] data) throws IOException {
-            return fill(block, data, 0, data.length);        }
-
-
-        @Override
-        public boolean fill(boolean block, ByteBuffer data, int len) throws IOException {
-            boolean result = fill(block, data.array(), data.arrayOffset() + data.position(), len);
-            if (result) {
-                data.position(data.position() + len);
-            }
-            return result;
-        }
-
-
         @Override
         public boolean fill(boolean block, byte[] data, int offset, int length) throws IOException {
             // Note: Block is ignored for this test class. Reads always block.
@@ -1003,7 +905,7 @@ public abstract class Http2TestBase extends TomcatBaseTest {
 
 
         @Override
-        public ByteBuffer startRequestBodyFrame(int streamId, int payloadSize, boolean endOfStream) {
+        public ByteBuffer startRequestBodyFrame(int streamId, int payloadSize) {
             lastStreamId = Integer.toString(streamId);
             bytesRead += payloadSize;
             if (traceBody) {
@@ -1056,15 +958,9 @@ public abstract class Http2TestBase extends TomcatBaseTest {
 
         @Override
         public void emitHeader(String name, String value) {
+            // Date headers will always change so use a hard-coded default
             if ("date".equals(name)) {
-                // Date headers will always change so use a hard-coded default
                 value = DEFAULT_DATE;
-            } else if ("etag".equals(name) && value.startsWith("W/\"")) {
-                // etag headers will vary depending on when the source was
-                // checked out, unpacked, copied etc so use the same default as
-                // for date headers
-                int startOfTime = value.indexOf('-');
-                value = value.substring(0, startOfTime + 1) + DEFAULT_TIME + "\"";
             }
             // Some header values vary so ignore them
             if (HEADER_IGNORED.equals(name)) {
@@ -1087,12 +983,6 @@ public abstract class Http2TestBase extends TomcatBaseTest {
         }
 
 
-        @Override
-        public void headersContinue(int payloadSize, boolean endOfHeaders) {
-            // NO-OP: Logging occurs per header
-        }
-
-
         @Override
         public void headersEnd(int streamId) {
             trace.append(streamId + "-HeadersEnd\n");
@@ -1256,7 +1146,7 @@ public abstract class Http2TestBase extends TomcatBaseTest {
             IOTools.flow(bais, resp.getOutputStream());
 
             // Check for trailer headers
-            String trailerValue = req.getHeader(TRAILER_HEADER_NAME);
+            String trailerValue = req.getTrailerFields().get(TRAILER_HEADER_NAME);
             if (trailerValue != null) {
                 resp.getOutputStream().write(trailerValue.getBytes(StandardCharsets.UTF_8));
             }
@@ -1325,24 +1215,6 @@ public abstract class Http2TestBase extends TomcatBaseTest {
     }
 
 
-    static class ParameterServlet extends HttpServlet {
-
-        private static final long serialVersionUID = 1L;
-
-        @Override
-        protected void doPost(HttpServletRequest req, HttpServletResponse resp)
-                throws ServletException, IOException {
-
-            Map<String,String[]> params = req.getParameterMap();
-
-            resp.setContentType("text/plain");
-            resp.setCharacterEncoding("UTF-8");
-
-            resp.getWriter().print(params.size());
-        }
-    }
-
-
     static class SettingValue {
         private final int setting;
         private final long value;
