@@ -16,27 +16,18 @@
  */
 package org.apache.tomcat.util.net;
 
-import java.io.ByteArrayOutputStream;
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
 import java.io.Reader;
-import java.util.Arrays;
-import java.util.concurrent.CountDownLatch;
-import java.util.concurrent.atomic.AtomicInteger;
 
-import javax.net.SocketFactory;
 import javax.net.ssl.HandshakeCompletedEvent;
 import javax.net.ssl.HandshakeCompletedListener;
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLSocket;
 import javax.net.ssl.SSLSocketFactory;
-import javax.servlet.ServletException;
-import javax.servlet.http.HttpServlet;
-import javax.servlet.http.HttpServletRequest;
-import javax.servlet.http.HttpServletResponse;
 
 import org.junit.Assert;
 import org.junit.Assume;
@@ -48,6 +39,7 @@ import org.apache.catalina.startup.TesterServlet;
 import org.apache.catalina.startup.Tomcat;
 import org.apache.catalina.startup.TomcatBaseTest;
 import org.apache.tomcat.util.buf.ByteChunk;
+import org.apache.tomcat.util.compat.TLS;
 import org.apache.tomcat.websocket.server.WsContextListener;
 
 /**
@@ -78,78 +70,6 @@ public class TestSsl extends TomcatBaseTest {
                 TesterSupport.getLastClientAuthRequestedIssuerCount() == 0);
     }
 
-    @Test
-    public void testPost() throws Exception {
-        final SocketFactory socketFactory = TesterSupport.configureClientSsl();
-
-        Tomcat tomcat = getTomcatInstance();
-        TesterSupport.initSsl(tomcat);
-
-        Context ctxt = tomcat.addContext("", null);
-        Tomcat.addServlet(ctxt, "post", new SimplePostServlet());
-        ctxt.addServletMappingDecoded("/post", "post");
-        tomcat.start();
-        int iterations = 8;
-        final CountDownLatch latch = new CountDownLatch(iterations);
-        final AtomicInteger errorCount = new AtomicInteger(0);
-        for (int i = 0; i < iterations; i++) {
-            new Thread() {
-                @Override
-                public void run() {
-                    try {
-                        SSLSocket socket = (SSLSocket) socketFactory.createSocket("localhost",
-                                getPort());
-
-                        OutputStream os = socket.getOutputStream();
-
-                        byte[] bytes = new byte[16 * 1024 * 1024]; // 16MB
-                        Arrays.fill(bytes, (byte) 1);
-
-                        os.write("POST /post HTTP/1.1\r\n".getBytes());
-                        os.write("Host: localhost\r\n".getBytes());
-                        os.write(("Content-Length: " + Integer.valueOf(bytes.length) + "\r\n\r\n").getBytes());
-                        // Write in 128KB blocks
-                        for (int i = 0; i < bytes.length / (128 * 1024); i++) {
-                            os.write(bytes, 0, 1024 * 128);
-                            Thread.sleep(10);
-                        }
-                        os.flush();
-
-                        InputStream is = socket.getInputStream();
-
-                        // Skip to the end of the headers
-                        byte[] endOfHeaders = "\r\n\r\n".getBytes();
-                        int found = 0;
-                        while (found != endOfHeaders.length) {
-                            if (is.read() == endOfHeaders[found]) {
-                                found++;
-                            } else {
-                                found = 0;
-                            }
-                        }
-
-                        for (int i = 0; i < bytes.length; i++) {
-                            int read = is.read();
-                            if (bytes[i] != read) {
-                                System.err.print("Byte in position [" + i + "] had value [" + read +
-                                        "] rather than " + Byte.toString(bytes[i]));
-                                errorCount.incrementAndGet();
-                                break;
-                            }
-                        }
-                    } catch (Exception e) {
-                        e.printStackTrace();
-                        errorCount.incrementAndGet();
-                    } finally {
-                        latch.countDown();
-                    }
-                }
-            }.start();
-        }
-        latch.await();
-        Assert.assertEquals(0, errorCount.get());
-    }
-
     @Test
     public void testKeyPass() throws Exception {
         TesterSupport.configureClientSsl();
@@ -191,13 +111,13 @@ public class TestSsl extends TomcatBaseTest {
         tomcat.start();
 
         SSLContext sslCtx;
-        if (TesterSupport.isDefaultTLSProtocolForTesting13(tomcat.getConnector())) {
+        if (TLS.isTlsv13Available()) {
             // Force TLS 1.2 if TLS 1.3 is available as JSSE's TLS 1.3
             // implementation doesn't support Post Handshake Authentication
             // which is required for this test to pass.
-            sslCtx = SSLContext.getInstance(Constants.SSL_PROTO_TLSv1_2);
+            sslCtx = SSLContext.getInstance("TLSv1.2");
         } else {
-            sslCtx = SSLContext.getInstance(Constants.SSL_PROTO_TLS);
+            sslCtx = SSLContext.getInstance("TLS");
         }
         sslCtx.init(null, TesterSupport.getTrustManagers(), null);
         SSLSocketFactory socketFactory = sslCtx.getSocketFactory();
@@ -217,17 +137,28 @@ public class TestSsl extends TomcatBaseTest {
 
         socket.startHandshake();
 
-        doRequest(os, r);
-        // Handshake complete appears to be called asynchronously
-        int wait = 0;
-        while (wait < 5000 && !listener.isComplete()) {
-            wait += 50;
-            Thread.sleep(50);
+        // One request should be sufficient
+        int requestCount = 0;
+        int listenerComplete = 0;
+        try {
+            while (requestCount < 10) {
+                requestCount++;
+                doRequest(os, r);
+                Assert.assertTrue("Checking no client issuer has been requested",
+                        TesterSupport.getLastClientAuthRequestedIssuerCount() == 0);
+                if (listener.isComplete() && listenerComplete == 0) {
+                    listenerComplete = requestCount;
+                }
+            }
+        } catch (AssertionError | IOException e) {
+            String message = "Failed on request number " + requestCount
+                    + " after startHandshake(). " + e.getMessage();
+            log.error(message, e);
+            Assert.fail(message);
         }
-        Assert.assertTrue("Checking no client issuer has been requested",
-                TesterSupport.getLastClientAuthRequestedIssuerCount() == 0);
+
         Assert.assertTrue(listener.isComplete());
-        System.out.println("Renegotiation completed after " + wait + " ms");
+        System.out.println("Renegotiation completed after " + listenerComplete + " requests");
     }
 
     private void doRequest(OutputStream os, Reader r) throws IOException {
@@ -276,28 +207,4 @@ public class TestSsl extends TomcatBaseTest {
             return complete;
         }
     }
-
-    public class SimplePostServlet extends HttpServlet {
-
-        private static final long serialVersionUID = 1L;
-
-        @Override
-        protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
-            ByteArrayOutputStream baos = new ByteArrayOutputStream();
-            byte[] in = new byte[1500];
-            InputStream input = req.getInputStream();
-            while (true) {
-                int n = input.read(in);
-                if (n > 0) {
-                    baos.write(in, 0, n);
-                } else {
-                    break;
-                }
-            }
-            byte[] out = baos.toByteArray();
-            resp.setContentLength(out.length);
-            resp.getOutputStream().write(out);
-        }
-
-    }
 }
