@@ -20,23 +20,17 @@ import java.io.File;
 import java.io.FileInputStream;
 import java.io.IOException;
 import java.io.InputStream;
-import java.net.InetAddress;
 import java.net.Socket;
-import java.net.UnknownHostException;
 import java.security.KeyStore;
-import java.security.NoSuchAlgorithmException;
 import java.security.Principal;
 import java.security.PrivateKey;
 import java.security.cert.CertificateException;
 import java.security.cert.X509Certificate;
-import java.util.Locale;
 
 import javax.net.ssl.KeyManager;
 import javax.net.ssl.KeyManagerFactory;
 import javax.net.ssl.SSLContext;
 import javax.net.ssl.SSLEngine;
-import javax.net.ssl.SSLSocket;
-import javax.net.ssl.SSLSocketFactory;
 import javax.net.ssl.TrustManager;
 import javax.net.ssl.TrustManagerFactory;
 import javax.net.ssl.X509ExtendedKeyManager;
@@ -48,8 +42,6 @@ import javax.servlet.http.HttpServlet;
 import javax.servlet.http.HttpServletRequest;
 import javax.servlet.http.HttpServletResponse;
 
-import org.junit.Assert;
-
 import org.apache.catalina.Context;
 import org.apache.catalina.authenticator.SSLAuthenticator;
 import org.apache.catalina.connector.Connector;
@@ -60,11 +52,10 @@ import org.apache.catalina.startup.Tomcat;
 import org.apache.tomcat.jni.Library;
 import org.apache.tomcat.jni.LibraryNotFoundError;
 import org.apache.tomcat.jni.SSL;
-import org.apache.tomcat.util.compat.JreCompat;
+import org.apache.tomcat.util.compat.JrePlatform;
 import org.apache.tomcat.util.descriptor.web.LoginConfig;
 import org.apache.tomcat.util.descriptor.web.SecurityCollection;
 import org.apache.tomcat.util.descriptor.web.SecurityConstraint;
-import org.apache.tomcat.util.net.SSLHostConfigCertificate.Type;
 
 public final class TesterSupport {
 
@@ -73,19 +64,16 @@ public final class TesterSupport {
     public static final String CA_JKS = SSL_DIR + CA_ALIAS + ".jks";
     public static final String CLIENT_ALIAS = "user1";
     public static final String CLIENT_JKS = SSL_DIR + CLIENT_ALIAS + ".jks";
-    public static final String LOCALHOST_EC_JKS = SSL_DIR + "localhost-ec.jks";
-    public static final String LOCALHOST_RSA_JKS = SSL_DIR + "localhost-rsa.jks";
-    public static final String LOCALHOST_KEYPASS_JKS = SSL_DIR + "localhost-rsa-copy1.jks";
+    public static final String LOCALHOST_JKS = SSL_DIR + "localhost.jks";
+    public static final String LOCALHOST_KEYPASS_JKS = SSL_DIR + "localhost-copy1.jks";
     public static final String JKS_PASS = "changeit";
     public static final String JKS_KEY_PASS = "tomcatpass";
     public static final String CA_CERT_PEM = SSL_DIR + CA_ALIAS + "-cert.pem";
-    public static final String LOCALHOST_EC_CERT_PEM = SSL_DIR + "localhost-ec-cert.pem";
-    public static final String LOCALHOST_EC_KEY_PEM = SSL_DIR + "localhost-ec-key.pem";
-    public static final String LOCALHOST_RSA_CERT_PEM = SSL_DIR + "localhost-rsa-cert.pem";
-    public static final String LOCALHOST_RSA_KEY_PEM = SSL_DIR + "localhost-rsa-key.pem";
+    public static final String LOCALHOST_CERT_PEM = SSL_DIR + "localhost-cert.pem";
+    public static final String LOCALHOST_KEY_PEM = SSL_DIR + "localhost-key.pem";
     public static final boolean OPENSSL_AVAILABLE;
     public static final int OPENSSL_VERSION;
-    public static final boolean TLSV13_AVAILABLE;
+    public static final String OPENSSL_ERROR;
 
     public static final String ROLE = "testrole";
 
@@ -96,24 +84,18 @@ public final class TesterSupport {
     static {
         boolean available = false;
         int version = 0;
+        String err = "";
         try {
             Library.initialize(null);
             available = true;
             version = SSL.version();
             Library.terminate();
         } catch (Exception | LibraryNotFoundError ex) {
-            // Ignore
+            err = ex.getMessage();
         }
         OPENSSL_AVAILABLE = available;
         OPENSSL_VERSION = version;
-
-        available = false;
-        try {
-            SSLContext.getInstance(Constants.SSL_PROTO_TLSv1_3);
-            available = true;
-        } catch (NoSuchAlgorithmException ex) {
-        }
-        TLSV13_AVAILABLE = available;
+        OPENSSL_ERROR = err;
     }
 
     public static boolean isOpensslAvailable() {
@@ -124,50 +106,54 @@ public final class TesterSupport {
         return OPENSSL_VERSION;
     }
 
-    public static boolean isTlsv13Available() {
-        return TLSV13_AVAILABLE;
-    }
-
     public static void initSsl(Tomcat tomcat) {
-        initSsl(tomcat, LOCALHOST_RSA_JKS, null, null);
+        initSsl(tomcat, LOCALHOST_JKS, null, null);
     }
 
     protected static void initSsl(Tomcat tomcat, String keystore,
             String keystorePass, String keyPass) {
 
-        Connector connector = tomcat.getConnector();
-        connector.setSecure(true);
-        Assert.assertTrue(connector.setProperty("SSLEnabled", "true"));
-
-        SSLHostConfig sslHostConfig = new SSLHostConfig();
-        SSLHostConfigCertificate certificate = new SSLHostConfigCertificate(sslHostConfig, Type.UNDEFINED);
-        sslHostConfig.addCertificate(certificate);
-        connector.addSslHostConfig(sslHostConfig);
-
         String protocol = tomcat.getConnector().getProtocolHandlerClassName();
         if (!protocol.contains("Apr")) {
+            Connector connector = tomcat.getConnector();
             String sslImplementation = System.getProperty("tomcat.test.sslImplementation");
             if (sslImplementation != null && !"${test.sslImplementation}".equals(sslImplementation)) {
                 StandardServer server = (StandardServer) tomcat.getServer();
                 AprLifecycleListener listener = new AprLifecycleListener();
                 listener.setSSLRandomSeed("/dev/urandom");
                 server.addLifecycleListener(listener);
-                connector.setAttribute("sslImplementationName", sslImplementation);
+                tomcat.getConnector().setAttribute("sslImplementationName", sslImplementation);
             }
-            sslHostConfig.setSslProtocol("tls");
-            certificate.setCertificateKeystoreFile(new File(keystore).getAbsolutePath());
-            sslHostConfig.setTruststoreFile(new File(CA_JKS).getAbsolutePath());
+            connector.setProperty("sslProtocol", "tls");
+            File keystoreFile =
+                new File(keystore);
+            connector.setAttribute("keystoreFile",
+                    keystoreFile.getAbsolutePath());
+            File truststoreFile = new File(CA_JKS);
+            connector.setAttribute("truststoreFile",
+                    truststoreFile.getAbsolutePath());
             if (keystorePass != null) {
-                certificate.setCertificateKeystorePassword(keystorePass);
+                connector.setAttribute("keystorePass", keystorePass);
             }
             if (keyPass != null) {
-                certificate.setCertificateKeyPassword(keyPass);
+                connector.setAttribute("keyPass", keyPass);
             }
         } else {
-            certificate.setCertificateFile(new File(LOCALHOST_RSA_CERT_PEM).getAbsolutePath());
-            certificate.setCertificateKeyFile(new File(LOCALHOST_RSA_KEY_PEM).getAbsolutePath());
-            sslHostConfig.setCaCertificateFile(new File(CA_CERT_PEM).getAbsolutePath());
+            File keystoreFile = new File(
+                    LOCALHOST_CERT_PEM);
+            tomcat.getConnector().setAttribute("SSLCertificateFile",
+                    keystoreFile.getAbsolutePath());
+            keystoreFile = new File(
+                    LOCALHOST_KEY_PEM);
+            tomcat.getConnector().setAttribute("SSLCertificateKeyFile",
+                    keystoreFile.getAbsolutePath());
+            keystoreFile = new File(
+                    CA_CERT_PEM);
+            tomcat.getConnector().setAttribute("SSLCACertificateFile",
+                    keystoreFile.getAbsolutePath());
         }
+        tomcat.getConnector().setSecure(true);
+        tomcat.getConnector().setProperty("SSLEnabled", "true");
     }
 
     protected static KeyManager[] getUser1KeyManagers() throws Exception {
@@ -194,19 +180,16 @@ public final class TesterSupport {
         return tmf.getTrustManagers();
     }
 
-    public static ClientSSLSocketFactory configureClientSsl() {
-        ClientSSLSocketFactory clientSSLSocketFactory = null;
+    protected static void configureClientSsl() {
         try {
-            SSLContext sc = SSLContext.getInstance(Constants.SSL_PROTO_TLS);
+            SSLContext sc = SSLContext.getInstance("TLS");
             sc.init(TesterSupport.getUser1KeyManagers(),
                     TesterSupport.getTrustManagers(),
                     null);
-            clientSSLSocketFactory = new ClientSSLSocketFactory(sc.getSocketFactory());
-            javax.net.ssl.HttpsURLConnection.setDefaultSSLSocketFactory(clientSSLSocketFactory);
+            javax.net.ssl.HttpsURLConnection.setDefaultSSLSocketFactory(sc.getSocketFactory());
         } catch (Exception e) {
             e.printStackTrace();
         }
-        return clientSSLSocketFactory;
     }
 
     private static KeyStore getKeyStore(String keystore) throws Exception {
@@ -218,10 +201,6 @@ public final class TesterSupport {
         return ks;
     }
 
-    protected static boolean isMacOs() {
-        return System.getProperty("os.name").toLowerCase(Locale.ENGLISH).startsWith("mac os x");
-    }
-
     protected static boolean isRenegotiationSupported(Tomcat tomcat) {
         String protocol = tomcat.getConnector().getProtocolHandlerClassName();
         if (protocol.contains("Apr")) {
@@ -238,7 +217,7 @@ public final class TesterSupport {
             // Disabled by default in 1.1.20 windows binary (2010-07-27)
             return false;
         }
-        if (protocol.contains("NioProtocol") || (protocol.contains("Nio2Protocol") && isMacOs())) {
+        if (protocol.contains("NioProtocol") || (protocol.contains("Nio2Protocol") && JrePlatform.IS_MAC_OS)) {
             // Doesn't work on all platforms - see BZ 56448.
             return false;
         }
@@ -257,16 +236,10 @@ public final class TesterSupport {
         /* When running on Java 11, TLSv1.3 is enabled by default. The JSSE
          * implementation of TLSv1.3 does not support
          * certificateVerification="optional", a setting on which these tests
-         * depend.
-         * Java 7 does not enable TLSv1.1 or TLS1.2 by default
-         *
-         * Ensure these tests pass with all JREs from Java 7 onwards.
+         * depend. Therefore, force these tests to use TLSv1.2 so that they pass
+         * when running on TLSv1.3.
          */
-        if (JreCompat.isJre8Available()) {
-            tomcat.getConnector().findSslHostConfigs()[0].setProtocols(Constants.SSL_PROTO_TLSv1_2);
-        } else {
-            tomcat.getConnector().findSslHostConfigs()[0].setProtocols(Constants.SSL_PROTO_TLSv1);
-        }
+        tomcat.getConnector().setProperty("sslEnabledProtocols", "TLSv1.2");
 
         // Need a web application with a protected and unprotected URL
         // No file system docBase required
@@ -587,112 +560,4 @@ public final class TesterSupport {
             }
         }
     }
-
-
-    public static class ClientSSLSocketFactory extends SSLSocketFactory {
-
-        private final SSLSocketFactory delegate;
-
-        private String[] ciphers = null;
-
-
-        public ClientSSLSocketFactory(SSLSocketFactory delegate) {
-            this.delegate = delegate;
-        }
-
-        /**
-         * Forces the use of the specified cipher.
-         *
-         * @param ciphers Array of standard JSSE names of ciphers to use
-         */
-        public void setCipher(String[] ciphers) {
-            this.ciphers = ciphers;
-        }
-
-        @Override
-        public Socket createSocket(Socket s, String host, int port, boolean autoClose) throws IOException {
-            Socket result = delegate.createSocket(s, host, port, autoClose);
-            reconfigureSocket(result);
-            return result;
-        }
-
-        @Override
-        public String[] getDefaultCipherSuites() {
-            return delegate.getDefaultCipherSuites();
-        }
-
-        @Override
-        public String[] getSupportedCipherSuites() {
-            return delegate.getSupportedCipherSuites();
-        }
-
-        @Override
-        public Socket createSocket(String host, int port) throws IOException, UnknownHostException {
-            Socket result = delegate.createSocket(host, port);
-            reconfigureSocket(result);
-            return result;
-        }
-
-        @Override
-        public Socket createSocket(InetAddress host, int port) throws IOException {
-            Socket result = delegate.createSocket(host, port);
-            reconfigureSocket(result);
-            return result;
-        }
-
-        @Override
-        public Socket createSocket(String host, int port, InetAddress localHost, int localPort)
-                throws IOException, UnknownHostException {
-            Socket result = delegate.createSocket(host, port, localHost, localPort);
-            reconfigureSocket(result);
-            return result;
-        }
-
-        @Override
-        public Socket createSocket(InetAddress address, int port, InetAddress localAddress, int localPort)
-                throws IOException {
-            Socket result = delegate.createSocket(address, port, localAddress, localPort);
-            reconfigureSocket(result);
-            return result;
-        }
-
-        private Socket reconfigureSocket(Socket socket) {
-            if (ciphers != null) {
-                ((SSLSocket) socket).setEnabledCipherSuites(ciphers);
-            }
-            return socket;
-        }
-    }
-
-
-    /*
-     * We want to use TLS 1.3 where we can but this requires TLS 1.3 to be
-     * supported on the client and the server.
-     */
-    public static String getDefaultTLSProtocolForTesting(Connector connector) {
-        // Clients always use JSSE
-        if (!TLSV13_AVAILABLE) {
-            // Client doesn't support TLS 1.3 so we have to use TLS 1.2
-            return Constants.SSL_PROTO_TLSv1_2;
-        }
-
-        if (connector.getProtocolHandlerClassName().contains("Apr")) {
-            // APR connector so OpenSSL is used for TLS.
-            if (SSL.version() >= 0x1010100f) {
-                return Constants.SSL_PROTO_TLSv1_3;
-            } else {
-                return Constants.SSL_PROTO_TLSv1_2;
-            }
-        } else {
-            // NIO or NIO2. Tests do not use JSSE+OpenSSL so JSSE will be used.
-            // Due to check above, it is known that TLS 1.3 is available
-            return Constants.SSL_PROTO_TLSv1_3;
-        }
-    }
-
-
-    public static boolean isDefaultTLSProtocolForTesting13(Connector connector) {
-        return Constants.SSL_PROTO_TLSv1_3.equals(
-                TesterSupport.getDefaultTLSProtocolForTesting(connector));
-    }
 }
